@@ -13,18 +13,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-STATUS_NEW = 'new'
+# SHARE AND GENERAL STATUSES
 STATUS_CREATING = 'creating'
 STATUS_DELETING = 'deleting'
 STATUS_DELETED = 'deleted'
 STATUS_ERROR = 'error'
 STATUS_ERROR_DELETING = 'error_deleting'
 STATUS_AVAILABLE = 'available'
-STATUS_ACTIVE = 'active'
 STATUS_INACTIVE = 'inactive'
-STATUS_OUT_OF_SYNC = 'out_of_sync'
-STATUS_UPDATING = 'updating'
-STATUS_UPDATING_MULTIPLE = 'updating_multiple'
 STATUS_MANAGING = 'manage_starting'
 STATUS_MANAGE_ERROR = 'manage_error'
 STATUS_UNMANAGING = 'unmanage_starting'
@@ -40,6 +36,37 @@ STATUS_SHRINKING_POSSIBLE_DATA_LOSS_ERROR = (
     'shrinking_possible_data_loss_error'
 )
 STATUS_REPLICATION_CHANGE = 'replication_change'
+STATUS_RESTORING = 'restoring'
+STATUS_REVERTING = 'reverting'
+STATUS_REVERTING_ERROR = 'reverting_error'
+
+# Access rule states
+ACCESS_STATE_QUEUED_TO_APPLY = 'queued_to_apply'
+ACCESS_STATE_QUEUED_TO_DENY = 'queued_to_deny'
+ACCESS_STATE_APPLYING = 'applying'
+ACCESS_STATE_DENYING = 'denying'
+ACCESS_STATE_ACTIVE = 'active'
+ACCESS_STATE_ERROR = 'error'
+ACCESS_STATE_DELETED = 'deleted'
+
+# Share instance "access_rules_status" field values
+SHARE_INSTANCE_RULES_SYNCING = 'syncing'
+SHARE_INSTANCE_RULES_ERROR = 'error'
+
+# States/statuses for multiple resources
+STATUS_NEW = 'new'
+STATUS_OUT_OF_SYNC = 'out_of_sync'
+STATUS_ACTIVE = 'active'
+
+ACCESS_RULES_STATES = (
+    ACCESS_STATE_QUEUED_TO_APPLY,
+    ACCESS_STATE_QUEUED_TO_DENY,
+    ACCESS_STATE_APPLYING,
+    ACCESS_STATE_DENYING,
+    ACCESS_STATE_ACTIVE,
+    ACCESS_STATE_ERROR,
+    ACCESS_STATE_DELETED,
+)
 
 TASK_STATE_MIGRATION_STARTING = 'migration_starting'
 TASK_STATE_MIGRATION_IN_PROGRESS = 'migration_in_progress'
@@ -81,18 +108,20 @@ TRANSITIONAL_STATUSES = (
     STATUS_MANAGING, STATUS_UNMANAGING,
     STATUS_EXTENDING, STATUS_SHRINKING,
     STATUS_MIGRATING, STATUS_MIGRATING_TO,
+    STATUS_RESTORING, STATUS_REVERTING,
 )
 
-UPDATING_RULES_STATUSES = (
-    STATUS_UPDATING,
-    STATUS_UPDATING_MULTIPLE,
+INVALID_SHARE_INSTANCE_STATUSES_FOR_ACCESS_RULE_UPDATES = (
+    TRANSITIONAL_STATUSES + (STATUS_ERROR,)
 )
 
 SUPPORTED_SHARE_PROTOCOLS = (
-    'NFS', 'CIFS', 'GLUSTERFS', 'HDFS', 'CEPHFS')
+    'NFS', 'CIFS', 'GLUSTERFS', 'HDFS', 'CEPHFS', 'MAPRFS')
 
 SECURITY_SERVICES_ALLOWED_TYPES = ['active_directory', 'ldap', 'kerberos']
 
+LIKE_FILTER = ['name~', 'description~']
+
 NFS_EXPORTS_FILE = '/etc/exports'
 NFS_EXPORTS_FILE_TEMP = '/var/lib/nfs/etab'
 
@@ -121,7 +150,7 @@ WINRM_PORTS = (
 )
 
 SERVICE_INSTANCE_SECGROUP_DATA = (
-    CIFS_PORTS + NFS_PORTS + SSH_PORTS + PING_PORTS + WINRM_PORTS)
+    CIFS_PORTS + NFS_PORTS + PING_PORTS + WINRM_PORTS)
 
 ACCESS_LEVEL_RW = 'rw'
 ACCESS_LEVEL_RO = 'ro'
@@ -153,6 +182,10 @@ REPLICA_STATE_ACTIVE = 'active'
 REPLICA_STATE_IN_SYNC = 'in_sync'
 REPLICA_STATE_OUT_OF_SYNC = 'out_of_sync'
 
+REPLICATION_TYPE_READABLE = 'readable'
+REPLICATION_TYPE_WRITABLE = 'writable'
+REPLICATION_TYPE_DR = 'dr'
+
 
 class ExtraSpecs(object):
 
@@ -160,19 +193,44 @@ class ExtraSpecs(object):
     DRIVER_HANDLES_SHARE_SERVERS = "driver_handles_share_servers"
     SNAPSHOT_SUPPORT = "snapshot_support"
     REPLICATION_TYPE_SPEC = "replication_type"
+    CREATE_SHARE_FROM_SNAPSHOT_SUPPORT = "create_share_from_snapshot_support"
+    REVERT_TO_SNAPSHOT_SUPPORT = "revert_to_snapshot_support"
+    MOUNT_SNAPSHOT_SUPPORT = "mount_snapshot_support"
 
     # Extra specs containers
     REQUIRED = (
         DRIVER_HANDLES_SHARE_SERVERS,
     )
-    UNDELETABLE = (
-        DRIVER_HANDLES_SHARE_SERVERS,
+
+    OPTIONAL = (
         SNAPSHOT_SUPPORT,
+        CREATE_SHARE_FROM_SNAPSHOT_SUPPORT,
+        REVERT_TO_SNAPSHOT_SUPPORT,
+        REPLICATION_TYPE_SPEC,
+        MOUNT_SNAPSHOT_SUPPORT,
     )
+
     # NOTE(cknight): Some extra specs are necessary parts of the Manila API and
-    # should be visible to non-admin users. UNDELETABLE specs are user-visible.
-    TENANT_VISIBLE = UNDELETABLE + (REPLICATION_TYPE_SPEC, )
+    # should be visible to non-admin users. REQUIRED specs are user-visible, as
+    # are a handful of community-agreed standardized OPTIONAL ones.
+    TENANT_VISIBLE = REQUIRED + OPTIONAL
+
     BOOLEAN = (
         DRIVER_HANDLES_SHARE_SERVERS,
         SNAPSHOT_SUPPORT,
+        CREATE_SHARE_FROM_SNAPSHOT_SUPPORT,
+        REVERT_TO_SNAPSHOT_SUPPORT,
+        MOUNT_SNAPSHOT_SUPPORT,
     )
+
+    # NOTE(cknight): Some extra specs are optional, but a nominal (typically
+    # False, but may be non-boolean) default value for each is still needed
+    # when creating shares.
+    INFERRED_OPTIONAL_MAP = {
+        SNAPSHOT_SUPPORT: False,
+        CREATE_SHARE_FROM_SNAPSHOT_SUPPORT: False,
+        REVERT_TO_SNAPSHOT_SUPPORT: False,
+        MOUNT_SNAPSHOT_SUPPORT: False,
+    }
+
+    REPLICATION_TYPES = ('writable', 'readable', 'dr')
