@@ -30,9 +30,6 @@ import six
 from manila.common import constants
 from manila import exception
 from manila.i18n import _
-from manila.i18n import _LE
-from manila.i18n import _LI
-from manila.i18n import _LW
 from manila.share import driver
 from manila.share.drivers.quobyte import jsonrpc
 
@@ -76,9 +73,15 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
         1.0.1   - Adds ensure_share() implementation.
         1.1     - Adds extend_share() and shrink_share() implementation.
         1.2     - Adds update_access() implementation and related methods
+        1.2.1   - Improved capacity calculation
+        1.2.2   - Minor optimizations
+        1.2.3   - Updated RPC layer for improved stability
+        1.2.4   - Fixed handling updated QB API error codes
+        1.2.5   - Fixed two quota handling bugs
+        1.2.6   - Fixed volume resize and jsonrpc code style bugs
     """
 
-    DRIVER_VERSION = '1.2'
+    DRIVER_VERSION = '1.2.6'
 
     def __init__(self, *args, **kwargs):
         super(QuobyteShareDriver, self).__init__(False, *args, **kwargs)
@@ -87,9 +90,8 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
                              or CONF.share_backend_name or 'Quobyte')
 
     def _fetch_existing_access(self, context, share):
-        volume_uuid = self._resolve_volume_name(
-            share['name'],
-            self._get_project_name(context, share['project_id']))
+        volume_uuid = self._resolve_volume_name(share['name'],
+                                                share['project_id'])
         result = self.rpc.call('getConfiguration', {})
         if result is None:
             raise exception.QBException(
@@ -121,7 +123,7 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
         try:
             self.rpc.call('getInformation', {})
         except Exception as exc:
-            LOG.error(_LE("Could not connect to API: %s"), exc)
+            LOG.error("Could not connect to API: %s", exc)
             raise exception.QBException(
                 _('Could not connect to API: %s') % exc)
 
@@ -141,18 +143,28 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
     def _get_capacities(self):
         result = self.rpc.call('getSystemStatistics', {})
 
-        total = float(result['total_logical_capacity'])
-        used = float(result['total_logical_usage'])
-        LOG.info(_LI('Read capacity of %(cap)s bytes and '
-                     'usage of %(use)s bytes from backend. '),
+        total = float(result['total_physical_capacity'])
+        used = float(result['total_physical_usage'])
+        LOG.info('Read capacity of %(cap)s bytes and '
+                 'usage of %(use)s bytes from backend. ',
                  {'cap': total, 'use': used})
         free = total - used
+        if free < 0:
+            free = 0  # no space available
+        free_replicated = free / self._get_qb_replication_factor()
         # floor numbers to nine digits (bytes)
         total = math.floor((total / units.Gi) * units.G) / units.G
-        free = math.floor((free / units.Gi) * units.G) / units.G
+        free = math.floor((free_replicated / units.Gi) * units.G) / units.G
 
         return total, free
 
+    def _get_qb_replication_factor(self):
+        result = self.rpc.call('getEffectiveVolumeConfiguration',
+                               {'configuration_name': self.
+                                configuration.quobyte_volume_configuration})
+        return int(result['configuration']['volume_metadata_configuration']
+                   ['replication_factor'])
+
     def check_for_setup_error(self):
         pass
 
@@ -168,16 +180,23 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
         return project_id
 
     def _resize_share(self, share, new_size):
-        # TODO(kaisers): check and update existing quota if already present
-        self.rpc.call('setQuota', {"consumer": {"type": 3,
-                                                "identifier": share["name"]},
-                                   "limits": {"type": 5, "value": new_size}})
+        newsize_bytes = new_size * units.Gi
+        self.rpc.call('setQuota', {"quotas": [
+            {"consumer":
+                [{"type": "VOLUME",
+                  "identifier": self._resolve_volume_name(share["name"],
+                                                          share['project_id']),
+                  "tenant_id": share["project_id"]}],
+             "limits": [{"type": "LOGICAL_DISK_SPACE",
+                        "value": newsize_bytes}]}
+        ]})
 
     def _resolve_volume_name(self, volume_name, tenant_domain):
         """Resolve a volume name to the global volume uuid."""
         result = self.rpc.call('resolveVolumeName', dict(
             volume_name=volume_name,
-            tenant_domain=tenant_domain))
+            tenant_domain=tenant_domain), [jsonrpc.ERROR_ENOENT,
+                                           jsonrpc.ERROR_ENTITY_NOT_FOUND])
         if result:
             return result['volume_uuid']
         return None  # not found
@@ -205,11 +224,14 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
             raise exception.QBException(
                 _('Quobyte driver only supports NFS shares'))
 
-        volume_uuid = self._resolve_volume_name(
-            share['name'],
-            self._get_project_name(context, share['project_id']))
+        volume_uuid = self._resolve_volume_name(share['name'],
+                                                share['project_id'])
 
         if not volume_uuid:
+            # create tenant, expect ERROR_GARBAGE_ARGS if it already exists
+            self.rpc.call('setTenant',
+                          dict(tenant=dict(tenant_id=share['project_id'])),
+                          expected_errors=[jsonrpc.ERROR_GARBAGE_ARGS])
             result = self.rpc.call('createVolume', dict(
                 name=share['name'],
                 tenant_domain=share['project_id'],
@@ -223,26 +245,27 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
             volume_uuid=volume_uuid,
             protocol='NFS'))
 
+        self._resize_share(share, share['size'])
+
         return '%(nfs_server_ip)s:%(nfs_export_path)s' % result
 
     def delete_share(self, context, share, share_server=None):
         """Delete the corresponding Quobyte volume."""
-        volume_uuid = self._resolve_volume_name(
-            share['name'],
-            self._get_project_name(context, share['project_id']))
+        volume_uuid = self._resolve_volume_name(share['name'],
+                                                share['project_id'])
         if not volume_uuid:
-            LOG.warning(_LW("No volume found for "
-                            "share %(project_id)s/%(name)s")
-                        % {"project_id": share['project_id'],
-                           "name": share['name']})
+            LOG.warning("No volume found for "
+                        "share %(project_id)s/%(name)s",
+                        {"project_id": share['project_id'],
+                         "name": share['name']})
             return
 
         if self.configuration.quobyte_delete_shares:
             self.rpc.call('deleteVolume', {'volume_uuid': volume_uuid})
-
-        self.rpc.call('exportVolume', dict(
-            volume_uuid=volume_uuid,
-            remove_export=True))
+        else:
+            self.rpc.call('exportVolume', {"volume_uuid": volume_uuid,
+                                           "remove_export": True,
+                                           })
 
     def ensure_share(self, context, share, share_server=None):
         """Invoked to ensure that share is exported.
@@ -257,11 +280,10 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
             the backend
         """
 
-        volume_uuid = self._resolve_volume_name(
-            share['name'],
-            self._get_project_name(context, share['project_id']))
+        volume_uuid = self._resolve_volume_name(share['name'],
+                                                share['project_id'])
 
-        LOG.debug("Ensuring Quobyte share %s" % share['name'])
+        LOG.debug("Ensuring Quobyte share %s", share['name'])
 
         if not volume_uuid:
             raise (exception.ShareResourceNotFound(
@@ -279,9 +301,8 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
             raise exception.InvalidShareAccess(
                 _('Quobyte driver only supports ip access control'))
 
-        volume_uuid = self._resolve_volume_name(
-            share['name'],
-            self._get_project_name(context, share['project_id']))
+        volume_uuid = self._resolve_volume_name(share['name'],
+                                                share['project_id'])
         ro = access['access_level'] == (constants.ACCESS_LEVEL_RO)
         call_params = {
             "volume_uuid": volume_uuid,
@@ -298,16 +319,15 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
                       self._get_project_name(context, share['project_id']))
             return
 
-        volume_uuid = self._resolve_volume_name(
-            share['name'],
-            self._get_project_name(context, share['project_id']))
+        volume_uuid = self._resolve_volume_name(share['name'],
+                                                share['project_id'])
         call_params = {
             "volume_uuid": volume_uuid,
             "remove_allow_ip": access['access_to']}
         self.rpc.call('exportVolume', call_params)
 
     def extend_share(self, ext_share, ext_size, share_server=None):
-        """Uses resize_share to extend a share.
+        """Uses _resize_share to extend a share.
 
         :param ext_share: Share model.
         :param ext_size: New size of share (new_size > share['size']).
@@ -316,7 +336,7 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
         self._resize_share(share=ext_share, new_size=ext_size)
 
     def shrink_share(self, shrink_share, shrink_size, share_server=None):
-        """Uses resize_share to shrink a share.
+        """Uses _resize_share to shrink a share.
 
         Quobyte uses soft quotas. If a shares current size is bigger than
         the new shrunken size no data is lost. Data can be continuously read
@@ -361,7 +381,7 @@ class QuobyteShareDriver(driver.ExecuteMixin, driver.ShareDriver,):
                 self._allow_access(context, share, a_rule)
         else:
             if not access_rules:
-                LOG.warning(_LW("No access rules provided in update_access."))
+                LOG.warning("No access rules provided in update_access.")
             else:
                 # Handling access rule recovery
                 existing_rules = self._fetch_existing_access(context, share)
