@@ -32,14 +32,13 @@ from manila.common import constants as const
 from manila import compute
 from manila import context
 from manila import exception
-from manila.i18n import _, _LW
+from manila.i18n import _
 from manila.network.linux import ip_lib
 from manila.network.neutron import api as neutron
 from manila import utils
 
 LOG = log.getLogger(__name__)
 NEUTRON_NAME = "neutron"
-NOVA_NAME = "nova"
 
 share_servers_handling_mode_opts = [
     cfg.StrOpt(
@@ -102,11 +101,6 @@ share_servers_handling_mode_opts = [
         help="Attach share server directly to share network. "
              "Used only with Neutron and "
              "if driver_handles_share_servers=True."),
-    cfg.StrOpt(
-        "service_instance_network_helper_type",
-        default=NEUTRON_NAME,
-        help="Allowed values are %s. " % [NOVA_NAME, NEUTRON_NAME] +
-             "Only used if driver_handles_share_servers=True."),
     cfg.StrOpt(
         "admin_network_id",
         help="ID of neutron network used to communicate with admin network,"
@@ -123,17 +117,17 @@ no_share_servers_handling_mode_opts = [
         "service_instance_name_or_id",
         help="Name or ID of service instance in Nova to use for share "
              "exports. Used only when share servers handling is disabled."),
-    cfg.StrOpt(
+    cfg.HostAddressOpt(
         "service_net_name_or_ip",
         help="Can be either name of network that is used by service "
              "instance within Nova to get IP address or IP address itself "
-             "for managing shares there. "
+             "(either IPv4 or IPv6) for managing shares there. "
              "Used only when share servers handling is disabled."),
-    cfg.StrOpt(
+    cfg.HostAddressOpt(
         "tenant_net_name_or_ip",
         help="Can be either name of network that is used by service "
              "instance within Nova to get IP address or IP address itself "
-             "for exporting shares. "
+             "(either IPv4 or IPv6) for exporting shares. "
              "Used only when share servers handling is disabled."),
 ]
 
@@ -152,6 +146,11 @@ common_opts = [
         "max_time_to_build_instance",
         default=300,
         help="Maximum time in seconds to wait for creating service instance."),
+    cfg.BoolOpt(
+        "limit_ssh_access",
+        default=False,
+        help="Block SSH connection to the service instance from other "
+             "networks than service network."),
 ]
 
 CONF = cfg.CONF
@@ -183,22 +182,12 @@ class ServiceInstanceManager(object):
         return CONF.get(key)
 
     def _get_network_helper(self):
-        network_helper_type = (
-            self.get_config_option(
-                "service_instance_network_helper_type").lower())
-        if network_helper_type == NEUTRON_NAME:
-            return NeutronNetworkHelper(self)
-        elif network_helper_type == NOVA_NAME:
-            return NovaNetworkHelper(self)
-        else:
-            raise exception.ManilaException(
-                _("Wrong value '%(provided)s' for config opt "
-                  "'service_instance_network_helper_type'. "
-                  "Allowed values are %(allowed)s.") % dict(
-                      provided=network_helper_type,
-                      allowed=[NOVA_NAME, NEUTRON_NAME]))
+        # Historically, there were multiple types of network helper,
+        # but currently the only network helper type is Neutron.
+        return NeutronNetworkHelper(self)
 
     def __init__(self, driver_config=None):
+
         super(ServiceInstanceManager, self).__init__()
         self.driver_config = driver_config
 
@@ -257,13 +246,13 @@ class ServiceInstanceManager(object):
             self.admin_context,
             self.get_config_option('service_instance_name_or_id'))
 
-        if netutils.is_valid_ipv4(data['service_net_name_or_ip']):
+        if netutils.is_valid_ip(data['service_net_name_or_ip']):
             data['private_address'] = [data['service_net_name_or_ip']]
         else:
             data['private_address'] = self._get_addresses_by_network_name(
                 data['service_net_name_or_ip'], data['instance'])
 
-        if netutils.is_valid_ipv4(data['tenant_net_name_or_ip']):
+        if netutils.is_valid_ip(data['tenant_net_name_or_ip']):
             data['public_address'] = [data['tenant_net_name_or_ip']]
         else:
             data['public_address'] = self._get_addresses_by_network_name(
@@ -283,13 +272,13 @@ class ServiceInstanceManager(object):
             'instance_id': data['instance']['id'],
         }
         for key in ('private_address', 'public_address'):
-            data[key + '_v4'] = None
+            data[key + '_first'] = None
             for address in data[key]:
-                if netutils.is_valid_ipv4(address):
-                    data[key + '_v4'] = address
+                if netutils.is_valid_ip(address):
+                    data[key + '_first'] = address
                     break
-        share_server['ip'] = data['private_address_v4']
-        share_server['public_address'] = data['public_address_v4']
+        share_server['ip'] = data['private_address_first']
+        share_server['public_address'] = data['public_address_first']
         return {'backend_details': share_server}
 
     def _get_addresses_by_network_name(self, net_name, server):
@@ -320,42 +309,68 @@ class ServiceInstanceManager(object):
             raise exception.ServiceInstanceException(msg)
         return net_ips[0]
 
-    @utils.synchronized(
-        "service_instance_get_or_create_security_group", external=True)
-    def _get_or_create_security_group(self, context, name=None,
-                                      description=None):
+    def _get_or_create_security_groups(self, context, name=None,
+                                       description=None,
+                                       allow_ssh_subnet=False):
         """Get or create security group for service_instance.
 
         :param context: context, that should be used
         :param name: this is used for selection/creation of sec.group
         :param description: this is used on sec.group creation step only
+        :param allow_ssh_subnet: subnet details to allow ssh connection from,
+         if not supplied ssh will be allowed from any host
         :returns: SecurityGroup -- security group instance from Nova
         :raises: exception.ServiceInstanceException.
         """
+
+        sgs = []
+        # Common security group
         name = name or self.get_config_option(
             "service_instance_security_group")
         if not name:
-            LOG.warning(_LW("Name for service instance security group is not "
-                            "provided. Skipping security group step."))
+            LOG.warning("Name for service instance security group is not "
+                        "provided. Skipping security group step.")
             return None
-        s_groups = [s for s in self.compute_api.security_group_list(context)
-                    if s.name == name]
+        if not description:
+            description = ("This security group is intended "
+                           "to be used by share service.")
+        sec_group_data = const.SERVICE_INSTANCE_SECGROUP_DATA
+        if not allow_ssh_subnet:
+            sec_group_data += const.SSH_PORTS
+
+        sgs.append(self._get_or_create_security_group(name, description,
+                                                      sec_group_data))
+        if allow_ssh_subnet:
+            if "cidr" not in allow_ssh_subnet or 'id' not in allow_ssh_subnet:
+                raise exception.ManilaException(
+                    "Unable to limit SSH access")
+            ssh_sg_name = "manila-service-subnet-{}".format(
+                allow_ssh_subnet["id"])
+            sgs.append(self._get_or_create_security_group(
+                ssh_sg_name, description,
+                const.SSH_PORTS, allow_ssh_subnet["cidr"]))
+        return sgs
+
+    @utils.synchronized(
+        "service_instance_get_or_create_security_group", external=True)
+    def _get_or_create_security_group(self, name,
+                                      description, sec_group_data,
+                                      cidr="0.0.0.0/0"):
+        s_groups = self.network_helper.neutron_api.security_group_list({
+            "name": name,
+        })['security_groups']
+        s_groups = [s for s in s_groups if s['name'] == name]
         if not s_groups:
-            # Creating security group
-            if not description:
-                description = "This security group is intended "\
-                              "to be used by share service."
             LOG.debug("Creating security group with name '%s'.", name)
-            sg = self.compute_api.security_group_create(
-                context, name, description)
-            for protocol, ports in const.SERVICE_INSTANCE_SECGROUP_DATA:
-                self.compute_api.security_group_rule_create(
-                    context,
-                    parent_group_id=sg.id,
+            sg = self.network_helper.neutron_api.security_group_create(
+                name, description)['security_group']
+            for protocol, ports in sec_group_data:
+                self.network_helper.neutron_api.security_group_rule_create(
+                    parent_group_id=sg['id'],
                     ip_protocol=protocol,
                     from_port=ports[0],
                     to_port=ports[1],
-                    cidr="0.0.0.0/0",
+                    cidr=cidr,
                 )
         elif len(s_groups) > 1:
             msg = _("Ambiguous security_groups.")
@@ -367,15 +382,15 @@ class ServiceInstanceManager(object):
     def ensure_service_instance(self, context, server):
         """Ensures that server exists and active."""
         if 'instance_id' not in server:
-            LOG.warning(_LW("Unable to check server existence since "
-                            "'instance_id' key is not set in share server "
-                            "backend details."))
+            LOG.warning("Unable to check server existence since "
+                        "'instance_id' key is not set in share server "
+                        "backend details.")
             return False
         try:
             inst = self.compute_api.server_get(self.admin_context,
                                                server['instance_id'])
         except exception.InstanceNotFound:
-            LOG.warning(_LW("Service instance %s does not exist."),
+            LOG.warning("Service instance %s does not exist.",
                         server['instance_id'])
             return False
         if inst['status'] == 'ACTIVE':
@@ -498,12 +513,14 @@ class ServiceInstanceManager(object):
         """Returns ID of service image for service vm creating."""
         service_image_name = self.get_config_option("service_image_name")
         images = [image.id for image in self.compute_api.image_list(context)
-                  if image.name == service_image_name]
+                  if image.name == service_image_name
+                  and image.status == 'active']
         if len(images) == 1:
             return images[0]
         elif not images:
             raise exception.ServiceInstanceException(
-                _("Image with name '%s' not found.") % service_image_name)
+                _("Image with name '%s' was not found or is not in "
+                  "'active' state.") % service_image_name)
         else:
             raise exception.ServiceInstanceException(
                 _("Found more than one image by name '%s'.") %
@@ -518,11 +535,11 @@ class ServiceInstanceManager(object):
             raise exception.ServiceInstanceException(
                 _('Neither service instance password nor key are available.'))
         if not key_path:
-            LOG.warning(_LW(
+            LOG.warning(
                 'No key path is available. May be non-existent key path is '
                 'provided. Check path_to_private_key (current value '
                 '%(private_path)s) and path_to_public_key (current value '
-                '%(public_path)s) in manila configuration file.'), dict(
+                '%(public_path)s) in manila configuration file.', dict(
                     private_path=self.path_to_private_key,
                     public_path=self.path_to_public_key))
         network_data = self.network_helper.setup_network(network_info)
@@ -556,34 +573,37 @@ class ServiceInstanceManager(object):
                 service_instance['id'],
                 self.max_time_to_build_instance)
 
-            security_group = self._get_or_create_security_group(context)
-            if security_group:
-                if self.network_helper.NAME == NOVA_NAME:
-                    # NOTE(vponomaryov): Nova-network allows to assign
-                    #                    secgroups only by names.
-                    sg_id = security_group.name
-                else:
-                    sg_id = security_group.id
+            if self.get_config_option("limit_ssh_access"):
+                try:
+                    service_subnet = network_data['service_subnet']
+                except KeyError:
+                    LOG.error(
+                        "Unable to limit ssh access to instance id: '%s'!",
+                        fail_safe_data['instance_id'])
+                    raise exception.ManilaException(
+                        "Unable to limit SSH access - "
+                        "invalid service subnet details provided")
+            else:
+                service_subnet = False
+
+            sec_groups = self._get_or_create_security_groups(
+                context, allow_ssh_subnet=service_subnet)
+
+            for sg in sec_groups:
+                sg_id = sg['id']
                 LOG.debug(
                     "Adding security group '%(sg)s' to server '%(si)s'.",
                     dict(sg=sg_id, si=service_instance["id"]))
                 self.compute_api.add_security_group_to_server(
                     context, service_instance["id"], sg_id)
 
-            if self.network_helper.NAME == NEUTRON_NAME:
-                ip = (network_data.get('service_port',
-                                       network_data.get(
-                                           'admin_port'))['fixed_ips'])
-                service_instance['ip'] = ip[0]['ip_address']
-                public_ip = (network_data.get(
-                    'public_port', network_data.get(
-                        'service_port'))['fixed_ips'])
-                service_instance['public_address'] = public_ip[0]['ip_address']
-            else:
-                net_name = self.network_helper.get_network_name(network_info)
-                service_instance['ip'] = self._get_server_ip(
-                    service_instance, net_name)
-                service_instance['public_address'] = service_instance['ip']
+            ip = (network_data.get('service_port',
+                                   network_data.get(
+                                       'admin_port'))['fixed_ips'])
+            service_instance['ip'] = ip[0]['ip_address']
+            public_ip = (network_data.get('public_port', network_data.get(
+                'service_port'))['fixed_ips'])
+            service_instance['public_address'] = public_ip[0]['ip_address']
 
         except Exception as e:
             e.detail_data = {'server_details': fail_safe_data}
@@ -668,7 +688,7 @@ class ServiceInstanceManager(object):
                 break
 
             LOG.debug("Waiting for instance %(instance_id)s to be active. "
-                      "Current status: %(instance_status)s." %
+                      "Current status: %(instance_status)s.",
                       dict(instance_id=instance_id,
                            instance_status=instance_status))
             time.sleep(1)
@@ -985,8 +1005,8 @@ class NeutronNetworkHelper(BaseNetworkhelper):
             addr_list = device.addr.list()
         except Exception as e:
             if 'does not exist' in six.text_type(e):
-                LOG.warning(_LW(
-                    "Device %s does not exist anymore.") % device.name)
+                LOG.warning(
+                    "Device %s does not exist anymore.", device.name)
             else:
                 raise
         for addr in addr_list:
@@ -1011,7 +1031,8 @@ class NeutronNetworkHelper(BaseNetworkhelper):
         elif not ports:
             port = self.neutron_api.create_port(
                 self.admin_project_id, network_id, subnet_id=subnet_id,
-                device_id=device_id, device_owner='manila:share', host_id=host)
+                device_id=device_id, device_owner='manila:share', host_id=host,
+                port_security_enabled=False)
         else:
             port = ports[0]
         return port
@@ -1087,50 +1108,3 @@ class NeutronNetworkHelper(BaseNetworkhelper):
         for subnet_id in service_network['subnets']:
             subnets.append(self.neutron_api.get_subnet(subnet_id))
         return subnets
-
-
-class NovaNetworkHelper(BaseNetworkhelper):
-    """Nova network helper for Manila service instances.
-
-    All security-group rules are applied to all interfaces of Nova VM
-    using Nova-network. In that case there is no need to create additional
-    service network. Only one thing should be satisfied - Manila host
-    should have access to all tenant networks.
-    This network helper does not create resources.
-    """
-
-    def __init__(self, service_instance_manager):
-        self.compute_api = service_instance_manager.compute_api
-        self.admin_context = service_instance_manager.admin_context
-
-    @property
-    def NAME(self):
-        return NOVA_NAME
-
-    def setup_network(self, network_info):
-        net = self._get_nova_network(network_info['nova_net_id'])
-        network_info['nics'] = [{'net-id': net['id']}]
-        return network_info
-
-    def get_network_name(self, network_info):
-        """Returns name of network for service instance."""
-        return self._get_nova_network(network_info['nova_net_id'])['label']
-
-    def teardown_network(self, server_details):
-        """Nothing to do. Placeholder."""
-
-    def setup_connectivity_with_service_instances(self):
-        """Nothing to do. Placeholder."""
-
-    def _get_nova_network(self, nova_network_id):
-        """Returns network to be used for service instance.
-
-        :param nova_network_id: string with id of network.
-        :returns: dict -- network data as dict
-        :raises: exception.ManilaException
-        """
-        if not nova_network_id:
-            raise exception.ManilaException(
-                _('Nova network for service instance is not provided.'))
-        net = self.compute_api.network_get(self.admin_context, nova_network_id)
-        return net
