@@ -17,6 +17,7 @@ import datetime
 
 import ddt
 import mock
+from oslo_config import cfg
 from oslo_utils import timeutils
 import webob
 
@@ -32,6 +33,8 @@ from manila import test
 from manila.tests.api import fakes
 from manila.tests import fake_notifier
 
+CONF = cfg.CONF
+
 
 def stub_share_type(id):
     specs = {
@@ -45,6 +48,7 @@ def stub_share_type(id):
     return dict(
         id=id,
         name='share_type_%s' % str(id),
+        description='description_%s' % str(id),
         extra_specs=specs,
         required_extra_specs={
             constants.ExtraSpecs.DRIVER_HANDLES_SHARE_SERVERS: "true",
@@ -60,6 +64,34 @@ def return_share_types_get_all_types(context, search_opts=None):
     )
 
 
+def stub_default_name():
+    return 'default_share_type'
+
+
+def stub_default_share_type(id):
+    return dict(
+        id=id,
+        name=stub_default_name(),
+        description='description_%s' % str(id),
+        required_extra_specs={
+            constants.ExtraSpecs.DRIVER_HANDLES_SHARE_SERVERS: "true",
+        }
+    )
+
+
+def return_all_share_types(context, search_opts=None):
+    mock_value = dict(
+        share_type_1=stub_share_type(1),
+        share_type_2=stub_share_type(2),
+        share_type_3=stub_default_share_type(3)
+    )
+    return mock_value
+
+
+def return_default_share_type(context, search_opts=None):
+    return stub_default_share_type(3)
+
+
 def return_empty_share_types_get_all_types(context, search_opts=None):
     return {}
 
@@ -88,25 +120,28 @@ def return_share_types_with_volumes_destroy(context, id):
     pass
 
 
-def return_share_types_create(context, name, specs, is_public):
+def return_share_types_create(context, name, specs, is_public, description):
     pass
 
 
 def make_create_body(name="test_share_1", extra_specs=None,
-                     spec_driver_handles_share_servers=True):
+                     spec_driver_handles_share_servers=True,
+                     description=None):
     if not extra_specs:
         extra_specs = {}
 
     if spec_driver_handles_share_servers is not None:
-        extra_specs[constants.ExtraSpecs.DRIVER_HANDLES_SHARE_SERVERS] =\
-            spec_driver_handles_share_servers
+        extra_specs[constants.ExtraSpecs.DRIVER_HANDLES_SHARE_SERVERS] = (
+            spec_driver_handles_share_servers)
 
     body = {
         "share_type": {
             "name": name,
-            "extra_specs": extra_specs
+            "extra_specs": extra_specs,
         }
     }
+    if description:
+        body["share_type"].update({"description": description})
 
     return body
 
@@ -115,7 +150,7 @@ def make_create_body(name="test_share_1", extra_specs=None,
 class ShareTypesAPITest(test.TestCase):
 
     def setUp(self):
-        super(self.__class__, self).setUp()
+        super(ShareTypesAPITest, self).setUp()
         self.flags(host='fake')
         self.controller = types.ShareTypesController()
         self.resource_name = self.controller.resource_name
@@ -123,14 +158,18 @@ class ShareTypesAPITest(test.TestCase):
                          mock.Mock(return_value=True))
         fake_notifier.reset()
         self.addCleanup(fake_notifier.reset)
-        self.mock_object(share_types, 'create',
-                         return_share_types_create)
-        self.mock_object(share_types, 'get_share_type_by_name',
-                         return_share_types_get_by_name)
-        self.mock_object(share_types, 'get_share_type',
-                         return_share_types_get_share_type)
-        self.mock_object(share_types, 'destroy',
-                         return_share_types_destroy)
+        self.mock_object(
+            share_types, 'create',
+            mock.Mock(side_effect=return_share_types_create))
+        self.mock_object(
+            share_types, 'get_share_type_by_name',
+            mock.Mock(side_effect=return_share_types_get_by_name))
+        self.mock_object(
+            share_types, 'get_share_type',
+            mock.Mock(side_effect=return_share_types_get_share_type))
+        self.mock_object(
+            share_types, 'destroy',
+            mock.Mock(side_effect=return_share_types_destroy))
 
     @ddt.data(True, False)
     def test_share_types_index(self, admin):
@@ -152,7 +191,7 @@ class ShareTypesAPITest(test.TestCase):
                 self.assertEqual('value1', entry['extra_specs'].get('key1'))
             else:
                 self.assertIsNone(entry['extra_specs'].get('key1'))
-            self.assertTrue('required_extra_specs' in entry)
+            self.assertIn('required_extra_specs', entry)
             required_extra_spec = entry['required_extra_specs'].get(
                 constants.ExtraSpecs.DRIVER_HANDLES_SHARE_SERVERS, '')
             self.assertEqual('true', required_extra_spec)
@@ -180,6 +219,9 @@ class ShareTypesAPITest(test.TestCase):
         self.assertEqual(2, len(res_dict))
         self.assertEqual('1', res_dict['share_type']['id'])
         self.assertEqual('share_type_1', res_dict['share_type']['name'])
+        expect = {constants.ExtraSpecs.DRIVER_HANDLES_SHARE_SERVERS: "true"}
+        self.assertEqual(expect,
+                         res_dict['share_type']['required_extra_specs'])
         policy.check_policy.assert_called_once_with(
             req.environ['manila.context'], self.resource_name, 'show')
 
@@ -217,18 +259,31 @@ class ShareTypesAPITest(test.TestCase):
             req.environ['manila.context'], self.resource_name, 'default')
 
     @ddt.data(
-        ('1.0', 'os-share-type-access'),
-        ('2.0', 'os-share-type-access'),
-        ('2.6', 'os-share-type-access'),
-        ('2.7', 'share_type_access'),
+        ('1.0', 'os-share-type-access', True),
+        ('1.0', 'os-share-type-access', False),
+        ('2.0', 'os-share-type-access', True),
+        ('2.0', 'os-share-type-access', False),
+        ('2.6', 'os-share-type-access', True),
+        ('2.6', 'os-share-type-access', False),
+        ('2.7', 'share_type_access', True),
+        ('2.7', 'share_type_access', False),
+        ('2.23', 'share_type_access', True),
+        ('2.23', 'share_type_access', False),
+        ('2.24', 'share_type_access', True),
+        ('2.24', 'share_type_access', False),
+        ('2.27', 'share_type_access', True),
+        ('2.27', 'share_type_access', False),
+        ('2.41', 'share_type_access', True),
+        ('2.41', 'share_type_access', False),
     )
     @ddt.unpack
-    def test_view_builder_show(self, version, prefix):
+    def test_view_builder_show(self, version, prefix, admin):
         view_builder = views_types.ViewBuilder()
 
         now = timeutils.utcnow().isoformat()
         raw_share_type = dict(
             name='new_type',
+            description='description_test',
             deleted=False,
             created_at=now,
             updated_at=now,
@@ -238,7 +293,8 @@ class ShareTypesAPITest(test.TestCase):
             id=42,
         )
 
-        request = fakes.HTTPRequest.blank("/v%s" % version[0], version=version)
+        request = fakes.HTTPRequest.blank("/v%s" % version[0], version=version,
+                                          use_admin_context=admin)
         request.headers['X-Openstack-Manila-Api-Version'] = version
 
         output = view_builder.show(request, raw_share_type)
@@ -251,46 +307,83 @@ class ShareTypesAPITest(test.TestCase):
             'required_extra_specs': {},
             'id': 42,
         }
-        self.assertDictMatch(output['share_type'], expected_share_type)
+        if self.is_microversion_ge(version, '2.24') and not admin:
+            for extra_spec in constants.ExtraSpecs.INFERRED_OPTIONAL_MAP:
+                expected_share_type['extra_specs'][extra_spec] = (
+                    constants.ExtraSpecs.INFERRED_OPTIONAL_MAP[extra_spec])
+        if self.is_microversion_ge(version, '2.41'):
+            expected_share_type['description'] = 'description_test'
 
-    def test_view_builder_list(self):
+        self.assertDictMatch(expected_share_type, output['share_type'])
+
+    @ddt.data(
+        ('1.0', 'os-share-type-access', True),
+        ('1.0', 'os-share-type-access', False),
+        ('2.0', 'os-share-type-access', True),
+        ('2.0', 'os-share-type-access', False),
+        ('2.6', 'os-share-type-access', True),
+        ('2.6', 'os-share-type-access', False),
+        ('2.7', 'share_type_access', True),
+        ('2.7', 'share_type_access', False),
+        ('2.23', 'share_type_access', True),
+        ('2.23', 'share_type_access', False),
+        ('2.24', 'share_type_access', True),
+        ('2.24', 'share_type_access', False),
+        ('2.27', 'share_type_access', True),
+        ('2.27', 'share_type_access', False),
+        ('2.41', 'share_type_access', True),
+        ('2.41', 'share_type_access', False),
+    )
+    @ddt.unpack
+    def test_view_builder_list(self, version, prefix, admin):
         view_builder = views_types.ViewBuilder()
 
+        extra_specs = {
+            constants.ExtraSpecs.SNAPSHOT_SUPPORT: True,
+            constants.ExtraSpecs.CREATE_SHARE_FROM_SNAPSHOT_SUPPORT: False,
+            constants.ExtraSpecs.REVERT_TO_SNAPSHOT_SUPPORT: True,
+            constants.ExtraSpecs.MOUNT_SNAPSHOT_SUPPORT: True,
+        }
+
         now = timeutils.utcnow().isoformat()
         raw_share_types = []
         for i in range(0, 10):
             raw_share_types.append(
                 dict(
                     name='new_type',
+                    description='description_test',
                     deleted=False,
                     created_at=now,
                     updated_at=now,
-                    extra_specs={},
+                    extra_specs=extra_specs,
                     required_extra_specs={},
                     deleted_at=None,
                     id=42 + i
                 )
             )
 
-        request = fakes.HTTPRequest.blank("/v2")
+        request = fakes.HTTPRequest.blank("/v%s" % version[0], version=version,
+                                          use_admin_context=admin)
         output = view_builder.index(request, raw_share_types)
 
         self.assertIn('share_types', output)
+        expected_share_type = {
+            'name': 'new_type',
+            'extra_specs': extra_specs,
+            '%s:is_public' % prefix: True,
+            'required_extra_specs': {},
+        }
+        if self.is_microversion_ge(version, '2.41'):
+            expected_share_type['description'] = 'description_test'
         for i in range(0, 10):
-            expected_share_type = {
-                'name': 'new_type',
-                'extra_specs': {},
-                'os-share-type-access:is_public': True,
-                'required_extra_specs': {},
-                'id': 42 + i,
-            }
-            self.assertDictMatch(output['share_types'][i],
-                                 expected_share_type)
+            expected_share_type['id'] = 42 + i
+            self.assertDictMatch(expected_share_type,
+                                 output['share_types'][i])
 
     @ddt.data(None, True, 'true', 'false', 'all')
     def test_parse_is_public_valid(self, value):
         result = self.controller._parse_is_public(value)
-        self.assertTrue(result in (True, False, None))
+        self.assertIn(result, (True, False, None))
 
     def test_parse_is_public_invalid(self):
         self.assertRaises(webob.exc.HTTPBadRequest,
@@ -310,28 +403,62 @@ class ShareTypesAPITest(test.TestCase):
                           req, '777')
         self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
 
+    def test_share_types_delete_in_use(self):
+
+        req = fakes.HTTPRequest.blank('/v2/fake/types/1')
+        self.assertEqual(0, len(fake_notifier.NOTIFICATIONS))
+        side_effect = exception.ShareTypeInUse(share_type_id='fake_id')
+        self.mock_object(share_types, 'destroy',
+                         mock.Mock(side_effect=side_effect))
+
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller._delete,
+                          req, 1)
+
     def test_share_types_with_volumes_destroy(self):
         req = fakes.HTTPRequest.blank('/v2/fake/types/1')
         self.assertEqual(0, len(fake_notifier.NOTIFICATIONS))
         self.controller._delete(req, 1)
         self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
 
-    @ddt.data(make_create_body("share_type_1"),
-              make_create_body(spec_driver_handles_share_servers="false"),
-              make_create_body(spec_driver_handles_share_servers="true"),
-              make_create_body(spec_driver_handles_share_servers="1"),
-              make_create_body(spec_driver_handles_share_servers="0"),
-              make_create_body(spec_driver_handles_share_servers="True"),
-              make_create_body(spec_driver_handles_share_servers="False"),
-              make_create_body(spec_driver_handles_share_servers="FalsE"))
-    def test_create(self, body):
-        req = fakes.HTTPRequest.blank('/v2/fake/types')
+    @ddt.data(
+        (make_create_body("share_type_1"), "2.24"),
+        (make_create_body(spec_driver_handles_share_servers=True), "2.24"),
+        (make_create_body(spec_driver_handles_share_servers=False), "2.24"),
+        (make_create_body("share_type_1"), "2.23"),
+        (make_create_body(spec_driver_handles_share_servers=True), "2.23"),
+        (make_create_body(spec_driver_handles_share_servers=False), "2.23"),
+        (make_create_body(description="description_1"), "2.41"))
+    @ddt.unpack
+    def test_create(self, body, version):
+
+        req = fakes.HTTPRequest.blank('/v2/fake/types', version=version)
         self.assertEqual(0, len(fake_notifier.NOTIFICATIONS))
-        res_dict = self.controller._create(req, body)
+
+        res_dict = self.controller.create(req, body)
+
         self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
         self.assertEqual(2, len(res_dict))
         self.assertEqual('share_type_1', res_dict['share_type']['name'])
         self.assertEqual('share_type_1', res_dict['volume_type']['name'])
+        if self.is_microversion_ge(version, '2.41'):
+            self.assertEqual(body['share_type']['description'],
+                             res_dict['share_type']['description'])
+            self.assertEqual(body['share_type']['description'],
+                             res_dict['volume_type']['description'])
+        for extra_spec in constants.ExtraSpecs.REQUIRED:
+            self.assertIn(extra_spec,
+                          res_dict['share_type']['required_extra_specs'])
+        expected_extra_specs = {
+            constants.ExtraSpecs.DRIVER_HANDLES_SHARE_SERVERS: True,
+        }
+        if self.is_microversion_lt(version, '2.24'):
+            expected_extra_specs[constants.ExtraSpecs.SNAPSHOT_SUPPORT] = True
+        expected_extra_specs.update(body['share_type']['extra_specs'])
+        share_types.create.assert_called_once_with(
+            mock.ANY, body['share_type']['name'],
+            expected_extra_specs, True,
+            description=body['share_type'].get('description'))
 
     @ddt.data(None,
               make_create_body(""),
@@ -342,13 +469,52 @@ class ShareTypesAPITest(test.TestCase):
               make_create_body(spec_driver_handles_share_servers=""),
               make_create_body(spec_driver_handles_share_servers=[]),
               )
-    def test_create_invalid_request(self, body):
-        req = fakes.HTTPRequest.blank('/v2/fake/types')
+    def test_create_invalid_request_1_0(self, body):
+        req = fakes.HTTPRequest.blank('/v2/fake/types', version="1.0")
         self.assertEqual(0, len(fake_notifier.NOTIFICATIONS))
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller._create, req, body)
+                          self.controller.create, req, body)
         self.assertEqual(0, len(fake_notifier.NOTIFICATIONS))
 
+    @ddt.data(*constants.ExtraSpecs.REQUIRED)
+    def test_create_invalid_request_2_23(self, required_extra_spec):
+
+        req = fakes.HTTPRequest.blank('/v2/fake/types', version="2.24")
+        self.assertEqual(0, len(fake_notifier.NOTIFICATIONS))
+        body = make_create_body("share_type_1")
+        del body['share_type']['extra_specs'][required_extra_spec]
+
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller.create, req, body)
+        self.assertEqual(0, len(fake_notifier.NOTIFICATIONS))
+
+    def test_create_already_exists(self):
+
+        side_effect = exception.ShareTypeExists(id='fake_id')
+        self.mock_object(share_types, 'create',
+                         mock.Mock(side_effect=side_effect))
+
+        req = fakes.HTTPRequest.blank('/v2/fake/types', version="2.24")
+        self.assertEqual(0, len(fake_notifier.NOTIFICATIONS))
+        body = make_create_body('share_type_1')
+
+        self.assertRaises(webob.exc.HTTPConflict,
+                          self.controller.create, req, body)
+        self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
+
+    def test_create_not_found(self):
+
+        self.mock_object(share_types, 'create',
+                         mock.Mock(side_effect=exception.NotFound))
+
+        req = fakes.HTTPRequest.blank('/v2/fake/types', version="2.24")
+        self.assertEqual(0, len(fake_notifier.NOTIFICATIONS))
+        body = make_create_body('share_type_1')
+
+        self.assertRaises(webob.exc.HTTPNotFound,
+                          self.controller.create, req, body)
+        self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
+
     def assert_share_type_list_equal(self, expected, observed):
         self.assertEqual(len(expected), len(observed))
         expected = sorted(expected, key=lambda item: item['id'])
@@ -356,11 +522,71 @@ class ShareTypesAPITest(test.TestCase):
         for d1, d2 in zip(expected, observed):
             self.assertEqual(d1['id'], d2['id'])
 
+    @ddt.data(('2.45', True), ('2.45', False),
+              ('2.46', True), ('2.46', False))
+    @ddt.unpack
+    def test_share_types_create_with_is_default_key(self, version, admin):
+        req = fakes.HTTPRequest.blank('/v2/fake/types',
+                                      version=version,
+                                      use_admin_context=admin)
+
+        body = make_create_body()
+        res_dict = self.controller.create(req, body)
+        if self.is_microversion_ge(version, '2.46'):
+            self.assertIn('is_default', res_dict['share_type'])
+            self.assertIs(False, res_dict['share_type']['is_default'])
+        else:
+            self.assertNotIn('is_default', res_dict['share_type'])
+
+    @ddt.data(('2.45', True), ('2.45', False),
+              ('2.46', True), ('2.46', False))
+    @ddt.unpack
+    def test_share_types_index_with_is_default_key(self, version, admin):
+        default_type_name = stub_default_name()
+        CONF.set_default("default_share_type", default_type_name)
+        self.mock_object(share_types, 'get_all_types',
+                         return_all_share_types)
+
+        req = fakes.HTTPRequest.blank('/v2/fake/types',
+                                      version=version,
+                                      use_admin_context=admin)
+
+        res_dict = self.controller.index(req)
+        self.assertEqual(3, len(res_dict['share_types']))
+        for res in res_dict['share_types']:
+            if self.is_microversion_ge(version, '2.46'):
+                self.assertIn('is_default', res)
+                expected = res['name'] == default_type_name
+                self.assertIs(res['is_default'], expected)
+            else:
+                self.assertNotIn('is_default', res)
+
+    @ddt.data(('2.45', True), ('2.45', False),
+              ('2.46', True), ('2.46', False))
+    @ddt.unpack
+    def test_share_types_default_with_is_default_key(self, version, admin):
+        default_type_name = stub_default_name()
+        CONF.set_default("default_share_type", default_type_name)
+        self.mock_object(share_types, 'get_default_share_type',
+                         return_default_share_type)
+
+        req = fakes.HTTPRequest.blank('/v2/fake/types/default_share_type',
+                                      version=version,
+                                      use_admin_context=admin)
+
+        res_dict = self.controller.default(req)
+        if self.is_microversion_ge(version, '2.46'):
+            self.assertIn('is_default', res_dict['share_type'])
+            self.assertIs(True, res_dict['share_type']['is_default'])
+        else:
+            self.assertNotIn('is_default', res_dict['share_type'])
+
 
 def generate_type(type_id, is_public):
     return {
         'id': type_id,
         'name': u'test',
+        'description': u'ds_test',
         'deleted': False,
         'created_at': datetime.datetime(2012, 1, 1, 1, 1, 1, 1),
         'updated_at': None,
@@ -429,13 +655,13 @@ class FakeRequest(object):
         return SHARE_TYPES[resource_id]
 
 
+@ddt.ddt
 class ShareTypeAccessTest(test.TestCase):
 
     def setUp(self):
-        super(self.__class__, self).setUp()
+        super(ShareTypeAccessTest, self).setUp()
         self.controller = types.ShareTypesController()
         self.req = FakeRequest()
-        self.context = self.req.environ['manila.context']
         self.mock_object(db, 'share_type_get', fake_share_type_get)
         self.mock_object(db, 'share_type_get_all', fake_share_type_get_all)
 
@@ -466,15 +692,23 @@ class ShareTypeAccessTest(test.TestCase):
         self.assertEqual(expected, result)
 
     def test_list_with_no_context(self):
-        req = fakes.HTTPRequest.blank('/v1/types/fake/types')
 
-        def fake_authorize(context, target=None, action=None):
-            raise exception.PolicyNotAuthorized(action='index')
+        req = fakes.HTTPRequest.blank('/v1/types/fake/types')
 
         self.assertRaises(webob.exc.HTTPForbidden,
                           self.controller.share_type_access,
                           req, 'fake')
 
+    def test_list_not_found(self):
+
+        side_effect = exception.ShareTypeNotFound(share_type_id='fake_id')
+        self.mock_object(share_types, 'get_share_type',
+                         mock.Mock(side_effect=side_effect))
+
+        self.assertRaises(webob.exc.HTTPNotFound,
+                          self.controller.share_type_access,
+                          self.req, 'fake')
+
     def test_list_type_with_admin_default_proj1(self):
         expected = {'share_types': [{'id': '0'}, {'id': '1'}]}
         req = fakes.HTTPRequest.blank('/v1/fake/types', use_admin_context=True)
@@ -592,6 +826,17 @@ class ShareTypeAccessTest(test.TestCase):
 
         self.assertEqual(202, result.status_code)
 
+    @ddt.data({'addProjectAccess': {'project': 'fake_project'}},
+              {'invalid': {'project': PROJ2_UUID}})
+    def test_add_project_access_bad_request(self, body):
+
+        req = fakes.HTTPRequest.blank('/v2/fake/types/2/action',
+                                      use_admin_context=True)
+
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller._add_project_access,
+                          req, '2', body)
+
     def test_add_project_access_with_no_admin_user(self):
         req = fakes.HTTPRequest.blank('/v2/fake/types/2/action',
                                       use_admin_context=False)
@@ -630,6 +875,32 @@ class ShareTypeAccessTest(test.TestCase):
         share_types.get_share_type.assert_called_once_with(
             mock.ANY, share_type_id)
 
+    def test_remove_project_access(self):
+
+        share_type = stub_share_type(2)
+        share_type['is_public'] = False
+        self.mock_object(share_types, 'get_share_type',
+                         mock.Mock(return_value=share_type))
+        self.mock_object(share_types, 'remove_share_type_access')
+        body = {'removeProjectAccess': {'project': PROJ2_UUID}}
+        req = fakes.HTTPRequest.blank('/v2/fake/types/2/action',
+                                      use_admin_context=True)
+
+        result = self.controller._remove_project_access(req, '2', body)
+
+        self.assertEqual(202, result.status_code)
+
+    @ddt.data({'removeProjectAccess': {'project': 'fake_project'}},
+              {'invalid': {'project': PROJ2_UUID}})
+    def test_remove_project_access_bad_request(self, body):
+
+        req = fakes.HTTPRequest.blank('/v2/fake/types/2/action',
+                                      use_admin_context=True)
+
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller._remove_project_access,
+                          req, '2', body)
+
     def test_remove_project_access_with_bad_access(self):
         def stub_remove_share_type_access(context, type_id, project_id):
             raise exception.ShareTypeAccessNotFound(share_type_id=type_id,
