@@ -14,12 +14,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import datetime
-import uuid
 
 from manila.api.openstack import api_version_request as api_version
 from manila.common import constants
 from manila.db.sqlalchemy import models
 from manila.tests.db import fakes as db_fakes
+from oslo_utils import uuidutils
 
 
 def fake_share(**kwargs):
@@ -31,15 +31,19 @@ def fake_share(**kwargs):
         'share_proto': 'fake_proto',
         'share_network_id': 'fake share network id',
         'share_server_id': 'fake share server id',
-        'share_type_id': 'fake share type id',
         'export_location': 'fake_location:/fake_share',
         'project_id': 'fake_project_uuid',
         'availability_zone': 'fake_az',
         'snapshot_support': 'True',
         'replication_type': None,
         'is_busy': False,
-        'consistency_group_id': 'fakecgid',
-        'instance': {'host': 'fakehost'},
+        'share_group_id': None,
+        'instance': {
+            'id': 'fake_share_instance_id',
+            'host': 'fakehost',
+            'share_type_id': '1',
+        },
+        'mount_snapshot_support': False,
     }
     share.update(kwargs)
     return db_fakes.FakeModel(share)
@@ -58,6 +62,7 @@ def fake_share_instance(base_share=None, **kwargs):
         'host': 'fakehost',
         'share_network_id': 'fakesharenetworkid',
         'share_server_id': 'fakeshareserverid',
+        'share_type_id': '1',
     }
 
     for attr in models.ShareInstance._proxified_properties:
@@ -76,7 +81,6 @@ def fake_share_type(**kwargs):
         'is_public': False,
         'extra_specs': {
             'driver_handles_share_servers': 'False',
-            'snapshot_support': 'True',
         }
     }
 
@@ -151,7 +155,10 @@ def fake_snapshot_instance(base_snapshot=None, as_primitive=False, **kwargs):
         'provider_location': 'i_live_here_actually',
         'share_name': 'fakename',
         'share_id': 'fakeshareinstanceid',
-        'share_instance': {'share_id': 'fakeshareid', },
+        'share_instance': {
+            'share_id': 'fakeshareid',
+            'share_type_id': '1',
+        },
         'share_instance_id': 'fakeshareinstanceid',
         'deleted': False,
         'updated_at': datetime.datetime(2016, 3, 21, 0, 5, 58),
@@ -230,7 +237,7 @@ def fake_access(**kwargs):
 
 def fake_replica(id=None, as_primitive=True, for_manager=False, **kwargs):
     replica = {
-        'id': id or str(uuid.uuid4()),
+        'id': id or uuidutils.generate_uuid(),
         'share_id': 'f0e4bb5e-65f0-11e5-9d70-feff819cdc9f',
         'deleted': False,
         'host': 'openstack@BackendZ#PoolA',
@@ -243,7 +250,7 @@ def fake_replica(id=None, as_primitive=True, for_manager=False, **kwargs):
         'export_locations': [{'path': 'path1'}, {'path': 'path2'}],
         'share_network_id': '4ccd5318-65f1-11e5-9d70-feff819cdc9f',
         'share_server_id': '53099868-65f1-11e5-9d70-feff819cdc9f',
-        'access_rules_status': 'out_of_sync',
+        'access_rules_status': constants.SHARE_INSTANCE_RULES_SYNCING,
     }
     if for_manager:
         replica.update({
@@ -253,11 +260,12 @@ def fake_replica(id=None, as_primitive=True, for_manager=False, **kwargs):
             'size': None,
             'display_name': None,
             'display_description': None,
+            'replication_type': None,
             'snapshot_id': None,
             'share_proto': None,
             'is_public': None,
-            'consistency_group_id': None,
-            'source_cgsnapshot_member_id': None,
+            'share_group_id': None,
+            'source_share_group_snapshot_member_id': None,
             'availability_zone': 'fake_az',
         })
     replica.update(kwargs)
@@ -278,7 +286,7 @@ def fake_replica_request_spec(as_primitive=True, **kwargs):
         'share_id': 'f0e4bb5e-65f0-11e5-9d70-feff819cdc9f',
         'snapshot_id': None,
         'share_type': 'fake_share_type',
-        'consistency_group': None,
+        'share_group': None,
         'active_replica_host': 'fake_active_replica_host',
         'all_replica_hosts': all_replica_hosts,
     }
