@@ -24,7 +24,8 @@ CONNECTION_INFO = {
     'transport_type': 'https',
     'port': 443,
     'username': 'admin',
-    'password': 'passw0rd'
+    'password': 'passw0rd',
+    'api_trace_pattern': '(.*)',
 }
 
 CLUSTER_NAME = 'fake_cluster'
@@ -32,12 +33,16 @@ REMOTE_CLUSTER_NAME = 'fake_cluster_2'
 CLUSTER_ADDRESS_1 = 'fake_cluster_address'
 CLUSTER_ADDRESS_2 = 'fake_cluster_address_2'
 VERSION = 'NetApp Release 8.2.1 Cluster-Mode: Fri Mar 21 14:25:07 PDT 2014'
-NODE_NAME = 'fake_node'
+VERSION_NO_DARE = 'NetApp Release 9.1.0: Tue May 10 19:30:23 2016 <1no-DARE>'
+VERSION_TUPLE = (9, 1, 0)
+NODE_NAME = 'fake_node1'
+NODE_NAMES = ('fake_node1', 'fake_node2')
 VSERVER_NAME = 'fake_vserver'
 VSERVER_NAME_2 = 'fake_vserver_2'
 ADMIN_VSERVER_NAME = 'fake_admin_vserver'
 NODE_VSERVER_NAME = 'fake_node_vserver'
 NFS_VERSIONS = ['nfs3', 'nfs4.0']
+ROOT_AGGREGATE_NAMES = ('root_aggr1', 'root_aggr2')
 ROOT_VOLUME_AGGREGATE_NAME = 'fake_root_aggr'
 ROOT_VOLUME_NAME = 'fake_root_volume'
 SHARE_AGGREGATE_NAME = 'fake_aggr1'
@@ -65,6 +70,8 @@ DELETED_EXPORT_POLICIES = {
         'deleted_manila_fake_policy_3',
     ],
 }
+QOS_POLICY_GROUP_NAME = 'fake_qos_policy_group_name'
+QOS_MAX_THROUGHPUT = '5000B/s'
 
 USER_NAME = 'fake_user'
 
@@ -73,6 +80,8 @@ VLAN = '1001'
 VLAN_PORT = 'e0a-1001'
 IP_ADDRESS = '10.10.10.10'
 NETMASK = '255.255.255.0'
+GATEWAY = '10.10.10.1'
+SUBNET = '10.10.10.0/24'
 NET_ALLOCATION_ID = 'fake_allocation_id'
 LIF_NAME_TEMPLATE = 'os_%(net_allocation_id)s'
 LIF_NAME = LIF_NAME_TEMPLATE % {'net_allocation_id': NET_ALLOCATION_ID}
@@ -94,6 +103,27 @@ NETWORK_INTERFACES = [{
     'home-port': VLAN_PORT
 }]
 
+NETWORK_INTERFACES_MULTIPLE = [
+    {
+        'interface_name': 'fake_interface',
+        'address': IP_ADDRESS,
+        'vserver': VSERVER_NAME,
+        'netmask': NETMASK,
+        'role': 'data',
+        'home-node': NODE_NAME,
+        'home-port': VLAN_PORT,
+    },
+    {
+        'interface_name': 'fake_interface_2',
+        'address': '10.10.12.10',
+        'vserver': VSERVER_NAME,
+        'netmask': NETMASK,
+        'role': 'data',
+        'home-node': NODE_NAME,
+        'home-port': PORT,
+    }
+]
+
 IPSPACES = [{
     'uuid': 'fake_uuid',
     'ipspace': IPSPACE_NAME,
@@ -117,6 +147,13 @@ EMS_MESSAGE = {
     'auto-support': 'false',
 }
 
+QOS_POLICY_GROUP = {
+    'policy-group': QOS_POLICY_GROUP_NAME,
+    'vserver': VSERVER_NAME,
+    'max-throughput': QOS_MAX_THROUGHPUT,
+    'num-workloads': 1,
+}
+
 NO_RECORDS_RESPONSE = etree.XML("""
   <results status="passed">
     <num-records>0</num-records>
@@ -127,6 +164,13 @@ PASSED_RESPONSE = etree.XML("""
   <results status="passed" />
 """)
 
+PASSED_FAILED_ITER_RESPONSE = etree.XML("""
+  <results status="passed">
+    <num-failed>0</num-failed>
+    <num-succeeded>1</num-succeeded>
+  </results>
+""")
+
 INVALID_GET_ITER_RESPONSE_NO_ATTRIBUTES = etree.XML("""
   <results status="passed">
     <num-records>1</num-records>
@@ -374,6 +418,7 @@ CIFS_SECURITY_SERVICE = {
     'type': 'active_directory',
     'password': 'fake_password',
     'user': 'fake_user',
+    'ou': 'fake_ou',
     'domain': 'fake_domain',
     'dns_ip': 'fake_dns_ip',
 }
@@ -412,6 +457,18 @@ SYSTEM_NODE_GET_ITER_RESPONSE = etree.XML("""
   </results>
 """ % NODE_NAME)
 
+SECUTITY_KEY_MANAGER_NVE_SUPPORT_RESPONSE_TRUE = etree.XML("""
+  <results status="passed">
+    <vol-encryption-supported>true</vol-encryption-supported>
+  </results>
+""")
+
+SECUTITY_KEY_MANAGER_NVE_SUPPORT_RESPONSE_FALSE = etree.XML("""
+  <results status="passed">
+    <vol-encryption-supported>false</vol-encryption-supported>
+  </results>
+""")
+
 NET_PORT_GET_ITER_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -791,34 +848,21 @@ AGGR_GET_NAMES_RESPONSE = etree.XML("""
     <attributes-list>
       <aggr-attributes>
         <aggr-raid-attributes>
-          <plexes>
-            <plex-attributes>
-              <plex-name>/%(aggr1)s/plex0</plex-name>
-              <raidgroups>
-                <raidgroup-attributes>
-                  <raidgroup-name>/%(aggr1)s/plex0/rg0</raidgroup-name>
-                </raidgroup-attributes>
-              </raidgroups>
-            </plex-attributes>
-          </plexes>
+        </aggr-raid-attributes>
+        <aggregate-name>%(root1)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+        </aggr-raid-attributes>
+        <aggregate-name>%(root2)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
         </aggr-raid-attributes>
         <aggregate-name>%(aggr1)s</aggregate-name>
       </aggr-attributes>
       <aggr-attributes>
         <aggr-raid-attributes>
-          <plexes>
-            <plex-attributes>
-              <plex-name>/%(aggr2)s/plex0</plex-name>
-              <raidgroups>
-                <raidgroup-attributes>
-                  <raidgroup-name>/%(aggr2)s/plex0/rg0</raidgroup-name>
-                </raidgroup-attributes>
-                <raidgroup-attributes>
-                  <raidgroup-name>/%(aggr2)s/plex0/rg1</raidgroup-name>
-                </raidgroup-attributes>
-              </raidgroups>
-            </plex-attributes>
-          </plexes>
         </aggr-raid-attributes>
         <aggregate-name>%(aggr2)s</aggregate-name>
       </aggr-attributes>
@@ -826,6 +870,8 @@ AGGR_GET_NAMES_RESPONSE = etree.XML("""
     <num-records>2</num-records>
   </results>
 """ % {
+    'root1': ROOT_AGGREGATE_NAMES[0],
+    'root2': ROOT_AGGREGATE_NAMES[1],
     'aggr1': SHARE_AGGREGATE_NAMES[0],
     'aggr2': SHARE_AGGREGATE_NAMES[1],
 })
@@ -1268,6 +1314,72 @@ AGGR_GET_ITER_SSC_RESPONSE = etree.XML("""
   </results>
 """ % {'aggr1': SHARE_AGGREGATE_NAMES[0]})
 
+AGGR_GET_ITER_ROOT_AGGR_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>true</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(root1)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>true</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(root2)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>false</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(aggr1)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>false</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(aggr2)s</aggregate-name>
+      </aggr-attributes>
+    </attributes-list>
+    <num-records>6</num-records>
+  </results>
+""" % {
+    'root1': ROOT_AGGREGATE_NAMES[0],
+    'root2': ROOT_AGGREGATE_NAMES[1],
+    'aggr1': SHARE_AGGREGATE_NAMES[0],
+    'aggr2': SHARE_AGGREGATE_NAMES[1],
+})
+
+AGGR_GET_ITER_NON_ROOT_AGGR_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>false</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(aggr1)s</aggregate-name>
+      </aggr-attributes>
+      <aggr-attributes>
+        <aggr-raid-attributes>
+          <has-local-root>false</has-local-root>
+          <has-partner-root>false</has-partner-root>
+        </aggr-raid-attributes>
+        <aggregate-name>%(aggr2)s</aggregate-name>
+      </aggr-attributes>
+    </attributes-list>
+    <num-records>6</num-records>
+  </results>
+""" % {
+    'aggr1': SHARE_AGGREGATE_NAMES[0],
+    'aggr2': SHARE_AGGREGATE_NAMES[1],
+})
+
 VOLUME_GET_NAME_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -1338,10 +1450,12 @@ VOLUME_MODIFY_ITER_ERROR_RESPONSE = etree.XML("""
   </results>
 """ % {'volume': SHARE_NAME, 'vserver': VSERVER_NAME})
 
+SNAPSHOT_ACCESS_TIME = '1466640058'
 SNAPSHOT_GET_ITER_NOT_BUSY_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
       <snapshot-info>
+        <access-time>%(access_time)s</access-time>
         <busy>false</busy>
         <name>%(snap)s</name>
         <volume>%(volume)s</volume>
@@ -1350,12 +1464,18 @@ SNAPSHOT_GET_ITER_NOT_BUSY_RESPONSE = etree.XML("""
     </attributes-list>
     <num-records>1</num-records>
   </results>
-""" % {'snap': SNAPSHOT_NAME, 'volume': SHARE_NAME, 'vserver': VSERVER_NAME})
+""" % {
+    'access_time': SNAPSHOT_ACCESS_TIME,
+    'snap': SNAPSHOT_NAME,
+    'volume': SHARE_NAME,
+    'vserver': VSERVER_NAME,
+})
 
 SNAPSHOT_GET_ITER_BUSY_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
       <snapshot-info>
+        <access-time>%(access_time)s</access-time>
         <busy>true</busy>
         <name>%(snap)s</name>
         <volume>%(volume)s</volume>
@@ -1369,7 +1489,12 @@ SNAPSHOT_GET_ITER_BUSY_RESPONSE = etree.XML("""
     </attributes-list>
     <num-records>1</num-records>
   </results>
-""" % {'snap': SNAPSHOT_NAME, 'volume': SHARE_NAME, 'vserver': VSERVER_NAME})
+""" % {
+    'access_time': SNAPSHOT_ACCESS_TIME,
+    'snap': SNAPSHOT_NAME,
+    'volume': SHARE_NAME,
+    'vserver': VSERVER_NAME,
+})
 
 SNAPSHOT_GET_ITER_NOT_UNIQUE_RESPONSE = etree.XML("""
   <results status="passed">
@@ -1456,6 +1581,24 @@ SNAPSHOT_GET_ITER_DELETED_RESPONSE = etree.XML("""
     'vserver': VSERVER_NAME,
 })
 
+SNAPSHOT_GET_ITER_SNAPMIRROR_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <snapshot-info>
+        <name>%(snap)s</name>
+        <volume>%(volume)s</volume>
+        <vserver>%(vserver)s</vserver>
+      </snapshot-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+
+""" % {
+    'snap': SNAPSHOT_NAME,
+    'volume': SHARE_NAME,
+    'vserver': VSERVER_NAME,
+})
+
 CIFS_SHARE_ACCESS_CONTROL_GET_ITER = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -1756,6 +1899,35 @@ GET_AGGREGATE_FOR_VOLUME_RESPONSE = etree.XML("""
     'share': SHARE_NAME
 })
 
+GET_VOLUME_FOR_ENCRYPTED_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <volume-attributes>
+        <encrypt>true</encrypt>
+        <volume-id-attributes>
+          <name>%(volume)s</name>
+          <owning-vserver-name>manila_svm</owning-vserver-name>
+        </volume-id-attributes>
+      </volume-attributes>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {'volume': SHARE_NAME})
+
+GET_VOLUME_FOR_ENCRYPTED_OLD_SYS_VERSION_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <volume-attributes>
+        <volume-id-attributes>
+          <name>%(volume)s</name>
+          <owning-vserver-name>manila_svm</owning-vserver-name>
+        </volume-id-attributes>
+      </volume-attributes>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {'volume': SHARE_NAME})
+
 EXPORT_RULE_GET_ITER_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -1872,6 +2044,36 @@ VOLUME_GET_ITER_JUNCTIONED_VOLUMES_RESPONSE = etree.XML("""
 """)
 
 VOLUME_GET_ITER_VOLUME_TO_MANAGE_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <volume-attributes>
+        <volume-id-attributes>
+          <containing-aggregate-name>%(aggr)s</containing-aggregate-name>
+          <junction-path>/%(volume)s</junction-path>
+          <name>%(volume)s</name>
+          <owning-vserver-name>%(vserver)s</owning-vserver-name>
+          <style>flex</style>
+          <type>rw</type>
+        </volume-id-attributes>
+        <volume-space-attributes>
+          <size>%(size)s</size>
+        </volume-space-attributes>
+        <volume-qos-attributes>
+          <policy-group-name>%(qos-policy-group-name)s</policy-group-name>
+        </volume-qos-attributes>
+      </volume-attributes>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {
+    'aggr': SHARE_AGGREGATE_NAME,
+    'vserver': VSERVER_NAME,
+    'volume': SHARE_NAME,
+    'size': SHARE_SIZE,
+    'qos-policy-group-name': QOS_POLICY_GROUP_NAME,
+})
+
+VOLUME_GET_ITER_NO_QOS_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
       <volume-attributes>
@@ -2058,7 +2260,187 @@ SNAPMIRROR_INITIALIZE_RESULT = etree.XML("""
   </results>
 """)
 
-FAKE_VOL_XML = """<volume-info xmlns='http://www.netapp.com/filer/admin'>
+VOLUME_MOVE_GET_ITER_RESULT = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <volume-move-info>
+        <cutover-action>retry_on_failure</cutover-action>
+        <details>Cutover Completed::Volume move job finishing move</details>
+        <estimated-completion-time>1481919246</estimated-completion-time>
+        <percent-complete>82</percent-complete>
+        <phase>finishing</phase>
+        <state>healthy</state>
+        <volume>%(volume)s</volume>
+        <vserver>%(vserver)s</vserver>
+      </volume-move-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {
+    'volume': SHARE_NAME,
+    'vserver': VSERVER_NAME,
+})
+
+PERF_OBJECT_COUNTER_TOTAL_CP_MSECS_LABELS = [
+    'SETUP', 'PRE_P0', 'P0_SNAP_DEL', 'P1_CLEAN', 'P1_QUOTA', 'IPU_DISK_ADD',
+    'P2V_INOFILE', 'P2V_INO_PUB', 'P2V_INO_PRI', 'P2V_FSINFO', 'P2V_DLOG1',
+    'P2V_DLOG2', 'P2V_REFCOUNT', 'P2V_TOPAA', 'P2V_DF_SCORES_SUB', 'P2V_BM',
+    'P2V_SNAP', 'P2V_DF_SCORES', 'P2V_VOLINFO', 'P2V_CONT', 'P2A_INOFILE',
+    'P2A_INO', 'P2A_DLOG1', 'P2A_HYA', 'P2A_DLOG2', 'P2A_FSINFO',
+    'P2A_IPU_BITMAP_GROW', 'P2A_REFCOUNT', 'P2A_TOPAA', 'P2A_HYABC', 'P2A_BM',
+    'P2A_SNAP', 'P2A_VOLINFO', 'P2_FLUSH', 'P2_FINISH', 'P3_WAIT',
+    'P3V_VOLINFO', 'P3A_VOLINFO', 'P3_FINISH', 'P4_FINISH', 'P5_FINISH',
+]
+
+PERF_OBJECT_COUNTER_LIST_INFO_WAFL_RESPONSE = etree.XML("""
+  <results status="passed">
+    <counters>
+      <counter-info>
+        <desc>No. of times 8.3 names are accessed per second.</desc>
+        <name>access_8_3_names</name>
+        <privilege-level>diag</privilege-level>
+        <properties>rate</properties>
+        <unit>per_sec</unit>
+      </counter-info>
+      <counter-info>
+        <desc>Array of counts of different types of CPs</desc>
+        <labels>
+          <label-info>wafl_timer generated CP</label-info>
+          <label-info>snapshot generated CP</label-info>
+          <label-info>wafl_avail_bufs generated CP</label-info>
+          <label-info>dirty_blk_cnt generated CP</label-info>
+          <label-info>full NV-log generated CP,back-to-back CP</label-info>
+          <label-info>flush generated CP,sync generated CP</label-info>
+          <label-info>deferred back-to-back CP</label-info>
+          <label-info>low mbufs generated CP</label-info>
+          <label-info>low datavecs generated CP</label-info>
+          <label-info>nvlog replay takeover time limit CP</label-info>
+        </labels>
+        <name>cp_count</name>
+        <privilege-level>diag</privilege-level>
+        <properties>delta</properties>
+        <type>array</type>
+        <unit>none</unit>
+      </counter-info>
+      <counter-info>
+        <base-counter>total_cp_msecs</base-counter>
+        <desc>Array of percentage time spent in different phases of CP</desc>
+        <labels>
+          <label-info>%(labels)s</label-info>
+        </labels>
+        <name>cp_phase_times</name>
+        <privilege-level>diag</privilege-level>
+        <properties>percent</properties>
+        <type>array</type>
+        <unit>percent</unit>
+      </counter-info>
+    </counters>
+  </results>
+""" % {'labels': ','.join(PERF_OBJECT_COUNTER_TOTAL_CP_MSECS_LABELS)})
+
+PERF_OBJECT_GET_INSTANCES_SYSTEM_RESPONSE_CMODE = etree.XML("""
+  <results status="passed">
+    <instances>
+      <instance-data>
+        <counters>
+          <counter-data>
+            <name>avg_processor_busy</name>
+            <value>5674745133134</value>
+          </counter-data>
+        </counters>
+        <name>system</name>
+        <uuid>%(node1)s:kernel:system</uuid>
+      </instance-data>
+      <instance-data>
+        <counters>
+          <counter-data>
+            <name>avg_processor_busy</name>
+            <value>4077649009234</value>
+          </counter-data>
+        </counters>
+        <name>system</name>
+        <uuid>%(node2)s:kernel:system</uuid>
+      </instance-data>
+    </instances>
+    <timestamp>1453412013</timestamp>
+  </results>
+""" % {'node1': NODE_NAMES[0], 'node2': NODE_NAMES[1]})
+
+PERF_OBJECT_GET_INSTANCES_SYSTEM_RESPONSE_7MODE = etree.XML("""
+  <results status="passed">
+    <timestamp>1454146292</timestamp>
+    <instances>
+      <instance-data>
+        <name>system</name>
+        <counters>
+          <counter-data>
+            <name>avg_processor_busy</name>
+            <value>13215732322</value>
+          </counter-data>
+        </counters>
+      </instance-data>
+    </instances>
+  </results>""")
+
+PERF_OBJECT_INSTANCE_LIST_INFO_ITER_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <instance-info>
+        <name>system</name>
+        <uuid>%(node)s:kernel:system</uuid>
+      </instance-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {'node': NODE_NAME})
+
+PERF_OBJECT_INSTANCE_LIST_INFO_RESPONSE = etree.XML("""
+  <results status="passed">
+    <instances>
+      <instance-info>
+        <name>processor0</name>
+      </instance-info>
+      <instance-info>
+        <name>processor1</name>
+      </instance-info>
+    </instances>
+  </results>""")
+
+NET_ROUTES_CREATE_RESPONSE = etree.XML("""
+  <results status="passed">
+    <result>
+      <net-vs-routes-info>
+        <address-family>ipv4</address-family>
+        <destination>%(subnet)s</destination>
+        <gateway>%(gateway)s</gateway>
+        <metric>20</metric>
+        <vserver>%(vserver)s</vserver>
+      </net-vs-routes-info>
+    </result>
+  </results>""" % {
+    'gateway': GATEWAY,
+    'vserver': VSERVER_NAME,
+    'subnet': SUBNET,
+})
+
+QOS_POLICY_GROUP_GET_ITER_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <qos-policy-group-info>
+        <max-throughput>%(max_througput)s</max-throughput>
+        <num-workloads>1</num-workloads>
+        <policy-group>%(qos_policy_group_name)s</policy-group>
+        <vserver>%(vserver)s</vserver>
+      </qos-policy-group-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>""" % {
+    'qos_policy_group_name': QOS_POLICY_GROUP_NAME,
+    'vserver': VSERVER_NAME,
+    'max_througput': QOS_MAX_THROUGHPUT,
+})
+
+FAKE_VOL_XML = """<volume-info>
     <name>open123</name>
     <state>online</state>
     <size-total>0</size-total>
@@ -2138,3 +2520,6 @@ FAKE_MANAGE_VOLUME = {
     'style': 'fake_style',
     'size': SHARE_SIZE,
 }
+
+FAKE_KEY_MANAGER_ERROR = "The onboard key manager is not enabled. To enable \
+                         it, run \"security key-manager setup\"."
