@@ -15,6 +15,7 @@
 
 import mock
 from oslo_config import cfg
+from oslo_utils import units
 import six
 
 from manila import context
@@ -29,7 +30,7 @@ from manila.tests import fake_share
 CONF = cfg.CONF
 
 
-def fake_rpc_handler(name, *args):
+def fake_rpc_handler(name, *args, **kwargs):
     if name == 'resolveVolumeName':
         return None
     elif name == 'createVolume':
@@ -114,7 +115,8 @@ class QuobyteShareDriverTestCase(test.TestCase):
         self.assertRaises(exception.QBException,
                           self._driver.do_setup, self._context)
 
-    def test_create_share_new_volume(self):
+    @mock.patch.object(quobyte.QuobyteShareDriver, "_resize_share")
+    def test_create_share_new_volume(self, qb_resize_mock):
         self._driver.rpc.call = mock.Mock(wraps=fake_rpc_handler)
 
         result = self._driver.create_share(self._context, self.share)
@@ -130,14 +132,32 @@ class QuobyteShareDriverTestCase(test.TestCase):
             )),
             mock.call('exportVolume',
                       dict(protocol='NFS', volume_uuid='voluuid'))])
+        qb_resize_mock.assert_called_once_with(self.share, self.share['size'])
 
-    def test_create_share_existing_volume(self):
+    @mock.patch.object(quobyte.QuobyteShareDriver, "_resize_share")
+    def test_create_share_existing_volume(self, qb_resize_mock):
         self._driver.rpc.call = mock.Mock(wraps=fake_rpc_handler)
 
         self._driver.create_share(self._context, self.share)
 
-        self._driver.rpc.call.assert_called_with(
-            'exportVolume', dict(protocol='NFS', volume_uuid='voluuid'))
+        resolv_params = {'tenant_domain': 'fake_project_uuid',
+                         'volume_name': 'fakename'}
+        sett_params = {'tenant': {'tenant_id': 'fake_project_uuid'}}
+        create_params = dict(
+            name='fakename',
+            tenant_domain='fake_project_uuid',
+            root_user_id='root',
+            root_group_id='root',
+            configuration_name='BASE')
+        self._driver.rpc.call.assert_has_calls([
+            mock.call('resolveVolumeName', resolv_params,
+                      [jsonrpc.ERROR_ENOENT, jsonrpc.ERROR_ENTITY_NOT_FOUND]),
+            mock.call('setTenant', sett_params,
+                      expected_errors=[jsonrpc.ERROR_GARBAGE_ARGS]),
+            mock.call('createVolume', create_params),
+            mock.call('exportVolume', dict(protocol='NFS',
+                                           volume_uuid='voluuid'))])
+        qb_resize_mock.assert_called_once_with(self.share, self.share['size'])
 
     def test_create_share_wrong_protocol(self):
         share = {'share_proto': 'WRONG_PROTOCOL'}
@@ -162,7 +182,8 @@ class QuobyteShareDriverTestCase(test.TestCase):
         resolv_params = {'volume_name': 'fakename',
                          'tenant_domain': 'fake_project_uuid'}
         self._driver.rpc.call.assert_has_calls([
-            mock.call('resolveVolumeName', resolv_params),
+            mock.call('resolveVolumeName', resolv_params,
+                      [jsonrpc.ERROR_ENOENT, jsonrpc.ERROR_ENTITY_NOT_FOUND]),
             mock.call('deleteVolume', {'volume_uuid': 'voluuid'})])
 
     def test_delete_share_existing_volume_disabled(self):
@@ -178,8 +199,7 @@ class QuobyteShareDriverTestCase(test.TestCase):
         self._driver.delete_share(self._context, self.share)
 
         self._driver.rpc.call.assert_called_with(
-            'exportVolume', {'volume_uuid': 'voluuid',
-                             'remove_export': True})
+            'exportVolume', {'volume_uuid': 'voluuid', 'remove_export': True})
 
     @mock.patch.object(quobyte.LOG, 'warning')
     def test_delete_share_nonexisting_volume(self, mock_warning):
@@ -192,7 +212,8 @@ class QuobyteShareDriverTestCase(test.TestCase):
         self._driver.delete_share(self._context, self.share)
 
         mock_warning.assert_called_with(
-            'No volume found for share fake_project_uuid/fakename')
+            'No volume found for share %(project_id)s/%(name)s',
+            {'project_id': 'fake_project_uuid', 'name': 'fakename'})
 
     def test_allow_access(self):
         def rpc_handler(name, *args):
@@ -276,8 +297,9 @@ class QuobyteShareDriverTestCase(test.TestCase):
 
         exp_params = {'volume_name': 'fake_vol_name',
                       'tenant_domain': 'fake_domain_name'}
-        self._driver.rpc.call.assert_called_with('resolveVolumeName',
-                                                 exp_params)
+        self._driver.rpc.call.assert_called_with(
+            'resolveVolumeName', exp_params,
+            [jsonrpc.ERROR_ENOENT, jsonrpc.ERROR_ENTITY_NOT_FOUND])
 
     def test_resolve_volume_name_NOENT(self):
         self._driver.rpc.call = mock.Mock(
@@ -286,6 +308,12 @@ class QuobyteShareDriverTestCase(test.TestCase):
         self.assertIsNone(
             self._driver._resolve_volume_name('fake_vol_name',
                                               'fake_domain_name'))
+        self._driver.rpc.call.assert_called_once_with(
+            'resolveVolumeName',
+            dict(volume_name='fake_vol_name',
+                 tenant_domain='fake_domain_name'),
+            [jsonrpc.ERROR_ENOENT, jsonrpc.ERROR_ENTITY_NOT_FOUND]
+        )
 
     def test_resolve_volume_name_other_error(self):
         self._driver.rpc.call = mock.Mock(
@@ -388,23 +416,31 @@ class QuobyteShareDriverTestCase(test.TestCase):
         mock_qsd_resize_share.assert_called_once_with(share=self.share,
                                                       new_size=2)
 
-    def test_resize_share(self):
+    @mock.patch.object(quobyte.QuobyteShareDriver, "_resolve_volume_name",
+                       return_value="fake_volume_uuid")
+    def test_resize_share(self, mock_qb_resolv):
         self._driver.rpc.call = mock.Mock(wraps=fake_rpc_handler)
+        manila_size = 7
+        newsize_bytes = manila_size * units.Gi
 
-        self._driver._resize_share(share=self.share, new_size=7)
+        self._driver._resize_share(share=self.share, new_size=manila_size)
 
         exp_params = {
-            "consumer": {
-                "type": 3,
-                "identifier": self.share["name"],
-            },
-            "limits": {
-                "type": 5,
-                "value": 7,
-            },
-        }
+            "quotas": [{
+                "consumer": [{
+                    "type": "VOLUME",
+                    "identifier": "fake_volume_uuid",
+                    "tenant_id": self.share["project_id"]
+                }],
+                "limits": [{
+                    "type": "LOGICAL_DISK_SPACE",
+                    "value": newsize_bytes,
+                }],
+            }]}
         self._driver.rpc.call.assert_has_calls([
             mock.call('setQuota', exp_params)])
+        mock_qb_resolv.assert_called_once_with(self.share['name'],
+                                               self.share['project_id'])
 
     @mock.patch.object(quobyte.QuobyteShareDriver,
                        "_resolve_volume_name",
