@@ -18,7 +18,9 @@ import os
 
 import ddt
 import mock
+from oslo_concurrency import processutils
 from oslo_config import cfg
+from oslo_utils import timeutils
 
 from manila.common import constants as const
 from manila import context
@@ -54,7 +56,12 @@ def fake_snapshot(**kwargs):
         'name': 'fakesnapshotname',
         'share_proto': 'NFS',
         'export_location': '127.0.0.1:/mnt/nfs/volume-00002',
-        'share': {'size': 1},
+        'share': {
+            'id': 'fakeid',
+            'name': 'fakename',
+            'size': 1,
+            'share_proto': 'NFS',
+        },
     }
     snapshot.update(kwargs)
     return db_fakes.FakeModel(snapshot)
@@ -82,7 +89,7 @@ class LVMShareDriverTestCase(test.TestCase):
         self._context = context.get_admin_context()
 
         CONF.set_default('lvm_share_volume_group', 'fakevg')
-        CONF.set_default('lvm_share_export_ip', '10.0.0.1')
+        CONF.set_default('lvm_share_export_ips', ['10.0.0.1', '10.0.0.2'])
         CONF.set_default('driver_handles_share_servers', False)
         CONF.set_default('reserved_share_percentage', 50)
 
@@ -103,7 +110,7 @@ class LVMShareDriverTestCase(test.TestCase):
         self.access = fake_access()
         self.snapshot = fake_snapshot()
         self.server = {
-            'public_address': self.fake_conf.lvm_share_export_ip,
+            'public_addresses': self.fake_conf.lvm_share_export_ips,
             'instance_id': 'LVM',
             'lock_name': 'manila_lvm',
         }
@@ -143,13 +150,33 @@ class LVMShareDriverTestCase(test.TestCase):
         self.assertRaises(exception.InvalidParameterValue,
                           self._driver.check_for_setup_error)
 
-    def test_check_for_setup_error_no_export_ip(self):
+    def test_check_for_setup_error_deprecated_export_ip(self):
         def exec_runner(*ignore_args, **ignore_kwargs):
             return '\n   fake1\n   fakevg\n   fake2\n', ''
 
         fake_utils.fake_execute_set_repliers([('vgs --noheadings -o name',
                                                exec_runner)])
-        CONF.set_default('lvm_share_export_ip', None)
+        CONF.set_default('lvm_share_export_ip', CONF.lvm_share_export_ips[0])
+        CONF.set_default('lvm_share_export_ips', None)
+        self.assertIsNone(self._driver.check_for_setup_error())
+
+    def test_check_for_setup_error_no_export_ips(self):
+        def exec_runner(*ignore_args, **ignore_kwargs):
+            return '\n   fake1\n   fakevg\n   fake2\n', ''
+
+        fake_utils.fake_execute_set_repliers([('vgs --noheadings -o name',
+                                               exec_runner)])
+        CONF.set_default('lvm_share_export_ips', None)
+        self.assertRaises(exception.InvalidParameterValue,
+                          self._driver.check_for_setup_error)
+
+    def test_check_for_setup_error_both_export_ip_and_ips(self):
+        def exec_runner(*ignore_args, **ignore_kwargs):
+            return '\n   fake1\n   fakevg\n   fake2\n', ''
+
+        fake_utils.fake_execute_set_repliers([('vgs --noheadings -o name',
+                                               exec_runner)])
+        CONF.set_default('lvm_share_export_ip', CONF.lvm_share_export_ips[0])
         self.assertRaises(exception.InvalidParameterValue,
                           self._driver.check_for_setup_error)
 
@@ -166,11 +193,12 @@ class LVMShareDriverTestCase(test.TestCase):
         self.assertEqual('/dev/mapper/fake--vg-fake--sharename', ret)
 
     def test_create_share(self):
-        self._helper_nfs.create_export.return_value = 'fakelocation'
+        CONF.set_default('lvm_share_mirrors', 0)
         self._driver._mount_device = mock.Mock()
+
         ret = self._driver.create_share(self._context, self.share,
                                         self.share_server)
-        CONF.set_default('lvm_share_mirrors', 0)
+
         self._driver._mount_device.assert_called_with(
             self.share, '/dev/mapper/fakevg-fakename')
         expected_exec = [
@@ -178,7 +206,7 @@ class LVMShareDriverTestCase(test.TestCase):
             'mkfs.ext4 /dev/mapper/fakevg-fakename',
         ]
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
-        self.assertEqual('fakelocation', ret)
+        self.assertEqual(self._helper_nfs.create_exports.return_value, ret)
 
     def test_create_share_from_snapshot(self):
         CONF.set_default('lvm_share_mirrors', 0)
@@ -200,6 +228,7 @@ class LVMShareDriverTestCase(test.TestCase):
         expected_exec = [
             'lvcreate -L 1G -n fakename fakevg',
             'mkfs.ext4 /dev/mapper/fakevg-fakename',
+            'tune2fs -U random %s' % mount_share,
             ("dd count=0 if=%s of=%s iflag=direct oflag=direct" %
              (mount_snapshot, mount_share)),
             ("dd if=%s of=%s count=1024 bs=1M iflag=direct oflag=direct" %
@@ -208,13 +237,13 @@ class LVMShareDriverTestCase(test.TestCase):
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
 
     def test_create_share_mirrors(self):
-
         share = fake_share(size='2048')
         CONF.set_default('lvm_share_mirrors', 2)
-        self._helper_nfs.create_export.return_value = 'fakelocation'
         self._driver._mount_device = mock.Mock()
+
         ret = self._driver.create_share(self._context, share,
                                         self.share_server)
+
         self._driver._mount_device.assert_called_with(
             share, '/dev/mapper/fakevg-fakename')
         expected_exec = [
@@ -222,7 +251,7 @@ class LVMShareDriverTestCase(test.TestCase):
             'mkfs.ext4 /dev/mapper/fakevg-fakename',
         ]
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
-        self.assertEqual('fakelocation', ret)
+        self.assertEqual(self._helper_nfs.create_exports.return_value, ret)
 
     def test_deallocate_container(self):
         expected_exec = ['lvremove -f fakevg/fakename']
@@ -258,21 +287,7 @@ class LVMShareDriverTestCase(test.TestCase):
                              self._driver.get_share_stats(refresh=True))
         self._driver._update_share_stats.assert_called_once_with()
 
-    def test_remove_export(self):
-        mount_path = self._get_mount_path(self.share)
-        self._os.path.exists.return_value = True
-
-        self._driver._remove_export(self._context, self.share)
-
-        expected_exec = [
-            "umount -f %s" % (mount_path,),
-        ]
-
-        self._os.path.exists.assert_called_with(mount_path)
-        self._os.rmdir.assert_called_with(mount_path)
-        self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
-
-    def test_remove_export_is_busy_error(self):
+    def test__unmount_device_is_busy_error(self):
         def exec_runner(*ignore_args, **ignore_kwargs):
             raise exception.ProcessExecutionError(stderr='device is busy')
         self._os.path.exists.return_value = True
@@ -283,44 +298,45 @@ class LVMShareDriverTestCase(test.TestCase):
         fake_utils.fake_execute_set_repliers([(expected_exec[0], exec_runner)])
 
         self.assertRaises(exception.ShareBusyException,
-                          self._driver._remove_export, self._context,
+                          self._driver._unmount_device,
                           self.share)
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
 
-    def test_remove_export_error(self):
+    def test__unmount_device_error(self):
         def exec_runner(*ignore_args, **ignore_kwargs):
             raise exception.ProcessExecutionError(stderr='fake error')
-
-        mount_path = self._get_mount_path(self.share)
-        expected_exec = [
-            "umount -f %s" % (mount_path),
-        ]
-        fake_utils.fake_execute_set_repliers([(expected_exec[0], exec_runner)])
-        self._os.path.exists.return_value = True
-        self._driver._remove_export(self._context, self.share)
-        self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
-
-    def test_remove_export_rmdir_error(self):
         mount_path = self._get_mount_path(self.share)
         self._os.path.exists.return_value = True
-        self.mock_object(self._os, 'rmdir', mock.Mock(side_effect=OSError))
+        cmd = "umount -f %s" % (mount_path)
+        fake_utils.fake_execute_set_repliers([(cmd, exec_runner)])
+        self.assertRaises(processutils.ProcessExecutionError,
+                          self._driver._unmount_device,
+                          self.share)
+        self._os.path.exists.assert_called_with(mount_path)
 
-        self._driver._remove_export(self._context, self.share)
-
-        expected_exec = [
-            "umount -f %s" % (mount_path,),
-        ]
-
+    def test__unmount_device_rmdir_error(self):
+        def exec_runner(*ignore_args, **ignore_kwargs):
+            raise exception.ProcessExecutionError(stderr='fake error')
+        mount_path = self._get_mount_path(self.share)
+        self._os.path.exists.return_value = True
+        cmd = "rmdir %s" % (mount_path)
+        fake_utils.fake_execute_set_repliers([(cmd, exec_runner)])
+        self.assertRaises(processutils.ProcessExecutionError,
+                          self._driver._unmount_device,
+                          self.share)
         self._os.path.exists.assert_called_with(mount_path)
-        self._os.rmdir.assert_called_with(mount_path)
-        self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
 
     def test_create_snapshot(self):
         self._driver.create_snapshot(self._context, self.snapshot,
                                      self.share_server)
+        mount_path = self._get_mount_path(self.snapshot)
         expected_exec = [
             ("lvcreate -L 1G --name fakesnapshotname --snapshot "
              "%s/fakename" % (CONF.lvm_share_volume_group,)),
+            "tune2fs -U random /dev/mapper/fakevg-%s" % self.snapshot['name'],
+            "mkdir -p " + mount_path,
+            "mount /dev/mapper/fakevg-fakesnapshotname " + mount_path,
+            "chmod 777 " + mount_path,
         ]
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
 
@@ -333,7 +349,7 @@ class LVMShareDriverTestCase(test.TestCase):
                                       self.share_server)
             self._driver._mount_device.assert_called_with(self.share,
                                                           device_name)
-            self._helper_nfs.create_export.assert_called_once_with(
+            self._helper_nfs.create_exports.assert_called_once_with(
                 self.server, self.share['name'], recreate=True)
 
     def test_delete_share(self):
@@ -342,7 +358,12 @@ class LVMShareDriverTestCase(test.TestCase):
         self._driver._delete_share(self._context, self.share)
 
     def test_delete_snapshot(self):
-        expected_exec = ['lvremove -f fakevg/fakesnapshotname']
+        mount_path = self._get_mount_path(self.snapshot)
+        expected_exec = [
+            'umount -f %s' % mount_path,
+            'rmdir %s' % mount_path,
+            'lvremove -f fakevg/fakesnapshotname',
+        ]
         self._driver.delete_snapshot(self._context, self.snapshot,
                                      self.share_server)
         self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
@@ -359,7 +380,7 @@ class LVMShareDriverTestCase(test.TestCase):
             mock.Mock(side_effect=exception.ProcessExecutionError))
 
         self._driver._delete_share(self._context, self.share)
-        self._helper_nfs.remove_export.assert_called_once_with(
+        self._helper_nfs.remove_exports.assert_called_once_with(
             self.server,
             self.share['name'])
 
@@ -380,6 +401,23 @@ class LVMShareDriverTestCase(test.TestCase):
                 self.server, self.share['name'],
                 access_rules, add_rules=add_rules, delete_rules=delete_rules))
 
+    @ddt.data(('1001::1001/129', None, False), ('1.1.1.256', None, False),
+              ('1001::1001', None, [6]), ('1.1.1.0', None, [4]),
+              (None, ['1001::1001', '1.1.1.0'], [6, 4]),
+              (None, ['1001::1001'], [6]), (None, ['1.1.1.0'], [4]),
+              (None, ['1001::1001/129', '1.1.1.0'], False))
+    @ddt.unpack
+    def test_get_configured_ip_versions(
+            self, configured_ip, configured_ips, configured_ip_version):
+        CONF.set_default('lvm_share_export_ip', configured_ip)
+        CONF.set_default('lvm_share_export_ips', configured_ips)
+        if configured_ip_version:
+            self.assertEqual(configured_ip_version,
+                             self._driver.get_configured_ip_versions())
+        else:
+            self.assertRaises(exception.InvalidInput,
+                              self._driver.get_configured_ip_versions)
+
     def test_mount_device(self):
         mount_path = self._get_mount_path(self.share)
         ret = self._driver._mount_device(self.share, 'fakedevice')
@@ -429,14 +467,16 @@ class LVMShareDriverTestCase(test.TestCase):
     def _get_mount_path(self, share):
         return os.path.join(CONF.lvm_share_export_root, share['name'])
 
-    def test_unmount_device(self):
+    def test__unmount_device(self):
         mount_path = self._get_mount_path(self.share)
+        self._os.path.exists.return_value = True
         self.mock_object(self._driver, '_execute')
         self._driver._unmount_device(self.share)
-        self._driver._execute.assert_any_call('umount', mount_path,
+        self._driver._execute.assert_any_call('umount', '-f', mount_path,
                                               run_as_root=True)
         self._driver._execute.assert_any_call('rmdir', mount_path,
                                               run_as_root=True)
+        self._os.path.exists.assert_called_with(mount_path)
 
     def test_extend_share(self):
         local_path = self._driver._get_local_path(self.share)
@@ -505,7 +545,10 @@ class LVMShareDriverTestCase(test.TestCase):
                                               'count=1024', 'bs=1M',
                                               run_as_root=True)
 
-    def test_update_share_stats(self):
+    @ddt.data(('1.1.1.1', 4), ('1001::1001', 6))
+    @ddt.unpack
+    def test_update_share_stats(self, configured_ip, version):
+        CONF.set_default('lvm_share_export_ip', configured_ip)
         self.mock_object(self._driver, 'get_share_server_pools',
                          mock.Mock(return_value='test-pool'))
 
@@ -513,7 +556,136 @@ class LVMShareDriverTestCase(test.TestCase):
         self.assertEqual('LVM', self._driver._stats['share_backend_name'])
         self.assertEqual('NFS_CIFS', self._driver._stats['storage_protocol'])
         self.assertEqual(50, self._driver._stats['reserved_percentage'])
-        self.assertIsNone(self._driver._stats['consistency_group_support'])
         self.assertTrue(self._driver._stats['snapshot_support'])
         self.assertEqual('LVMShareDriver', self._driver._stats['driver_name'])
         self.assertEqual('test-pool', self._driver._stats['pools'])
+        self.assertEqual(version == 4, self._driver._stats['ipv4_support'])
+        self.assertEqual(version == 6, self._driver._stats['ipv6_support'])
+
+    def test_revert_to_snapshot(self):
+        mock_update_access = self.mock_object(self._helper_nfs,
+                                              'update_access')
+        self._driver.revert_to_snapshot(self._context, self.snapshot,
+                                        [], [], self.share_server)
+        snap_lv = "%s/fakesnapshotname" % (CONF.lvm_share_volume_group)
+        share_lv = "%s/fakename" % (CONF.lvm_share_volume_group)
+        share_mount_path = self._get_mount_path(self.snapshot['share'])
+        snapshot_mount_path = self._get_mount_path(self.snapshot)
+        expected_exec = [
+            ('umount -f %s' % snapshot_mount_path),
+            ("rmdir %s" % snapshot_mount_path),
+            ("umount -f %s" % share_mount_path),
+            ("rmdir %s" % share_mount_path),
+            ("lvconvert --merge %s" % snap_lv),
+            ("lvcreate -L 1G --name fakesnapshotname --snapshot %s" %
+                share_lv),
+            ('tune2fs -U random /dev/mapper/%s-fakesnapshotname' %
+                CONF.lvm_share_volume_group),
+            ("mkdir -p %s" % share_mount_path),
+            ("mount /dev/mapper/%s-fakename %s" %
+                (CONF.lvm_share_volume_group, share_mount_path)),
+            ("chmod 777 %s" % share_mount_path),
+            ("mkdir -p %s" % snapshot_mount_path),
+            ("mount /dev/mapper/fakevg-fakesnapshotname "
+             "%s" % snapshot_mount_path),
+            ("chmod 777 %s" % snapshot_mount_path),
+        ]
+        self.assertEqual(expected_exec, fake_utils.fake_execute_get_log())
+        self.assertEqual(4, mock_update_access.call_count)
+
+    def test_snapshot_update_access(self):
+        access_rules = [{
+            'access_type': 'ip',
+            'access_to': '1.1.1.1',
+            'access_level': 'ro',
+        }]
+
+        add_rules = [{
+            'access_type': 'ip',
+            'access_to': '2.2.2.2',
+            'access_level': 'ro',
+        }]
+
+        delete_rules = [{
+            'access_type': 'ip',
+            'access_to': '3.3.3.3',
+            'access_level': 'ro',
+        }]
+
+        self._driver.snapshot_update_access(self._context, self.snapshot,
+                                            access_rules, add_rules,
+                                            delete_rules)
+
+        (self._driver._helpers[self.snapshot['share']['share_proto']].
+            update_access.assert_called_once_with(
+            self.server, self.snapshot['name'],
+            access_rules, add_rules=add_rules, delete_rules=delete_rules))
+
+    @mock.patch.object(timeutils, 'utcnow', mock.Mock(
+                       return_value='fake_date'))
+    def test_update_share_usage_size(self):
+        mount_path = self._get_mount_path(self.share)
+        self._os.path.exists.return_value = True
+        self.mock_object(
+            self._driver,
+            '_execute',
+            mock.Mock(return_value=(
+                "Mounted on                    Used "
+                + mount_path + "               1G", None)))
+
+        update_shares = self._driver.update_share_usage_size(
+            self._context, [self.share, ])
+        self._os.path.exists.assert_called_with(mount_path)
+        self.assertEqual(
+            [{'id': 'fakeid', 'used_size': '1',
+              'gathered_at': 'fake_date'}],
+            update_shares)
+        self._driver._execute.assert_called_once_with(
+            'df', '-l', '--output=target,used',
+            '--block-size=g')
+
+    @mock.patch.object(timeutils, 'utcnow', mock.Mock(
+                       return_value='fake_date'))
+    def test_update_share_usage_size_multiple_share(self):
+        share1 = fake_share(id='fakeid_get_fail', name='get_fail')
+        share2 = fake_share(id='fakeid_success', name='get_success')
+        share3 = fake_share(id='fakeid_not_exist', name='get_not_exist')
+
+        mount_path2 = self._get_mount_path(share2)
+        mount_path3 = self._get_mount_path(share3)
+        self._os.path.exists.side_effect = [True, True, False]
+        self.mock_object(
+            self._driver,
+            '_execute',
+            mock.Mock(return_value=(
+                "Mounted on                    Used "
+                + mount_path2 + "               1G", None)))
+
+        update_shares = self._driver.update_share_usage_size(
+            self._context, [share1, share2, share3])
+        self._os.path.exists.assert_called_with(mount_path3)
+        self.assertEqual(
+            [{'gathered_at': 'fake_date',
+              'id': 'fakeid_success', 'used_size': '1'}],
+            update_shares)
+        self._driver._execute.assert_called_with(
+            'df', '-l', '--output=target,used',
+            '--block-size=g')
+
+    def test_update_share_usage_size_fail(self):
+        def _fake_exec(*args, **kwargs):
+            raise exception.ProcessExecutionError(stderr="error")
+
+        self.mock_object(self._driver, '_execute', _fake_exec)
+        self.assertRaises(exception.ProcessExecutionError,
+                          self._driver.update_share_usage_size,
+                          self._context,
+                          [self.share])
+
+    def test_get_backend_info(self):
+        backend_info = self._driver.get_backend_info(self._context)
+
+        self.assertEqual(
+            {'export_ips': ','.join(self.server['public_addresses']),
+             'db_version': mock.ANY},
+            backend_info)
